"""
A burner-stabilized, premixed methane/air flat flame with multicomponent
transport properties and a specified temperature profile.
"""

import cantera as ct
import pandas as pd


print(ct.__version__)
print(ct.__file__)

path = "./"

################################################################
# parameter values
p = ct.one_atm  # pressure
tburner = 300.0  # burner temperature
#mdot = 0.04  # kg/m^2/s
u_inlet = 0.0826 # m/s
comp = 'C2H4:0.133, O2:0.193, AR:0.674'  # premixed gas composition

# The solution domain is chosen to be 2 cm
width = 0.02 # m

loglevel = 1  # amount of diagnostic output (0 to 5)
refine_grid = True  # 'True' to enable refinement

################ create the gas object ########################
#
# This object will be used to evaluate all thermodynamic, kinetic, and
# transport properties. It is created with two transport managers, to enable
# switching from mixture-averaged to multicomponent transport on the last
# solution.
gas = ct.Solution(path + 'appel_2000.xml')

# set its state to that of the unburned gas at the burner
gas.TPX = tburner, p, comp

# get rho in order to get mdot
rho_inlet = gas.density_mass
mdot = rho_inlet * u_inlet # [m.s-1].[kg.m-3] = [kg.m-2.s-1] 

# create the BurnerFlame object.
f = ct.BurnerFlame(gas=gas, width=width)

# set the mass flow rate at the burner
f.burner.mdot = mdot

# read temperature vs. position data from a file.
T_profile = pd.read_csv(path + "Zhao_T_profile.csv")
# The file is assumed to have one z, T pair per line, separated by a comma.
#zloc, tvalues = np.genfromtxt('tdata.dat', delimiter=',', comments='#').T
zloc = T_profile["x"]
tvalues = T_profile["T"]
zloc /= max(zloc)

# set the temperature profile to the values read in
# imposed on the flame object (IdealGasFlow), not on f (onedim.BurnerFlame) which
# is set at the initial guess: very coarse grid and higher T. 
f.flame.set_fixed_temp_profile(zloc, tvalues)

# show the initial estimate for the solution
f.show_solution()

# don't solve the energy equation
f.energy_enabled = False

# first solve the flame with mixture-averaged transport properties
f.transport_model = 'Mix'
#f.set_refine_criteria(ratio=3.0, slope=0.3, curve=1)
f.set_refine_criteria(ratio=10.0, slope=0.08, curve=0.01)

f.solve(loglevel, refine_grid)
f.save(path + 'outputs/c2h4_flame_fixed_T.xml','burnerFlame', 'solution with mixture-averaged transport')

f.write_csv(path + 'outputs/c2h4_flame_fixed_T.csv', quiet=False, species = "Y")

#sol = pd.read_csv(path + "c2h4_flame_fixed_T.csv")
#plt.plot(sol["z (m)"], sol["T (K)"])
#plt.plot(T_profile["x"],T_profile["T"])

#z = sol["z (m)"]


