library(readr)  # for read_csv()
library(dplyr)  # for mutate()
library(tidyr)  # for unnest()
library(purrr)  # for map(), reduce()
library(magrittr)

library(ggplot2)
library(gganimate)
# from https://github.com/thomasp85/gganimate


# find all file names ending in .csv 
#files <- dir(pattern = "*.csv")
#data_path <- "/Users/bouaniche/nps/samples/gas/outputs/psd/"   # path to the data
data_path <- getwd()   # path to the data

files <- dir(data_path, pattern = "*.dat") # get file names
# delete .dat files that are not psd...dat

clusters_path <- paste(data_path,"/psdClusters", sep = "")
clusters_files <- dir(clusters_path, pattern = "*.dat")

#data <- files %>%
#  map(read.table) %>%
#  reduce(rbind)        # reduce with rbind into one dataframe
#data

psd_t <- files %>%
  # read in all the files, appending the path before the filename
  map(~ read.table(file.path(data_path, .))) %>% 
  reduce(rbind)

psd_t <- as_tibble(psd_t)

# colnames for analytical

#colnames(psd_t) <- c("mAvg", "np", "Nv", "ndm", "ndmStoch", "NvR", "deltam", 
#                      "mInf", "mSup", "mAvgVar", "mAvgVarStoch", "mAvgVarR", "t")

# colnames for HomogeneousGasPhase (soot)

colnames(psd_t) <- c("mAvgj", "dAvg", "npj", "Nv", "nd", "ndExpNorm", 
                     "alphaHacaBini", "dcjNano", "kfCoef", "betaSelfAggloSpherej", 
                     "betaSelfAggloDcj", "betaSelfAggloHarmonic", "betaCondSpherej", 
"betaCondDcj", "betaCondharmonic", "surfSphere", "surfFractal", "kn", "nm", "ndmjStoch", 
"NvR", "deltamj", "mInfj", "mSupj", "mAvgVar", "mAvgVarStoch", "mAvgVarR", "t")

getDfromMassNano <- function(m){
  rhoSoot <- 1.8 #[g/cm^3]
  pi <- 3.14159265
  v <- m/rhoSoot   # [cm3]  m_mAvg in [g]
  d3 <- v*6.0/pi
  dcm <- d3^(1/3)  # [in cm]
  dnm <- dcm * 1.0e7   # [in nm]
  return(dnm)
}

psd_t <- psd_t %>%
  dplyr::mutate(dAvgVar = sapply(mAvgVar, getDfromMassNano))

psd_clusters <- clusters_files %>%
  map(~ read.table(file.path(clusters_path, .))) %>% 
  reduce(rbind)

psd_clusters <- as_tibble(psd_clusters)

colnames(psd_clusters) <- c("mAvgVar", "nm", "deltam", "t", "dAvgVar", "nd")

psd_clusters <- psd_clusters %>%
  mutate(Nv = nm*deltam, dataType = rep("clusters", nrow(psd_clusters)))

psd_clusters_selected <- psd_clusters %>%
  select(mAvgVar, dAvgVar, Nv, nd, t, dataType)

psd_t <- psd_t %>%
  mutate(dataType = rep("sections",nrow(psd_t)))

psd_t_selected <- psd_t %>%
  select(mAvgVar, dAvgVar, Nv, nd, t, dataType)

#psd_t_selected <- psd_t %>%
#  select(mAvgVar, Nvj, ndj, t, dataType, dAvgj, npj, mInfj, mSupj)

psd_all <- bind_rows(psd_t_selected, psd_clusters_selected)
#psd_all <- psd_t_selected


# First, check if no significant negative value.
# Then, For vizualisation purpose, set negative/null values of n(v) at 1e-10
check_neg <- psd_all %>%
  filter(nd <= 0)

check_high_neg <- psd_all %>%
  filter(nd < -1e2)

check_NA <- psd_all %>%
  dplyr::filter(is.na(nd))

psd_cleaned <- psd_all

psd_cleaned[is.na(psd_cleaned$nd), "nd"] <- 1e-10
psd_cleaned[is.na(psd_cleaned$Nv), "Nv"] <- 1e-10
psd_cleaned[psd_cleaned$nd <= 0, "nd"] <- 1e-10
psd_cleaned[psd_cleaned$Nv <= 0, "Nv"] <- 1e-10

# only sections
psd_sections <- psd_cleaned %>%
  dplyr::filter(dataType == "sections")

ggplot(psd_sections, aes(dAvgVar, nd, shape = dataType)) +
  geom_point(alpha = 0.7, show.legend = TRUE, aes(colour = dataType)) +
  #geom_line() +
  scale_x_log10(limits = c(7e-1,25)) +
  scale_y_log10(limits = c(1e9, 1e12)) +
  scale_color_manual(values=c("black")) +
  theme_bw() +
  # Here comes the gganimate specific bits
  transition_time(t) +
  labs(x = 'd [nm]', y = expression(paste('n(d) [', cm^{-3}, "]", sep = ""))) +
  ease_aes('linear') +
  theme(axis.text.x = element_text(size = 20),
        axis.text.y = element_text(size = 20),
        axis.title.x = element_text(size = 20),
        axis.title.y = element_text(size = 20),
        legend.text = element_text(size = 20),
        legend.title = element_text(size = 20),
        legend.position = 'bottom')

# save
anim_name <- "soot_sections.gif"
anim_path <- data_path
anim_save(filename = anim_name, path = anim_path)