//
//  HomogeneousGasPhase.cpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 30/08/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#include "HomogeneousGasPhase.hpp"

#include <iostream>
#include <string>
#include <vector>
#include <cmath>

using namespace std;

const double HomogeneousGasPhase::avogadro = 6.024e23;
const double HomogeneousGasPhase::WA1 = 78.11; // [g/mol]
const double HomogeneousGasPhase::WA2 = 128.1705;
const double HomogeneousGasPhase::WA3 = 178.23;
const double HomogeneousGasPhase::WA4 = 202.25;
const double HomogeneousGasPhase::WC2H2 = 26.04;
const double HomogeneousGasPhase::WO2 = 31.99880;
const double HomogeneousGasPhase::WOH = 17.008;
const double HomogeneousGasPhase::WH2O = 18.01528;
const double HomogeneousGasPhase::WH2 = 2.01588;
const double HomogeneousGasPhase::WH = 1.007940;


HomogeneousGasPhase::HomogeneousGasPhase()
{
    
}

HomogeneousGasPhase::HomogeneousGasPhase(string pathProject, string pathTarget, string dataFilename,string labelsFilename)
{
    vector<string> labels = readLabels(pathProject, pathTarget , labelsFilename);
    
    vector<vector<double> > dataArray = readDataArray(pathProject, pathTarget, dataFilename, labelsFilename);
    
    int indext = dataIndex(labels, "t");
    int indexT = dataIndex(labels, "T");
    int indexYPAHnucl = dataIndex(labels, "YPAHnucl");
    int indexYA1 = dataIndex(labels, "YA1");
    int indexYA2 = dataIndex(labels, "YA2");
    int indexYA3 = dataIndex(labels, "YA3");
    int indexYA4 = dataIndex(labels, "YA4");
    int indexYC2H2 = dataIndex(labels, "YC2H2");
    int indexYO2 = dataIndex(labels, "YO2");
    int indexYOH = dataIndex(labels, "YOH");
    int indexYA4imposed = dataIndex(labels, "YA4imposed");
    int indexwmA4 = dataIndex(labels, "wmA4");
    
    int indexx = dataIndex(labels, "x");
    int indexu = dataIndex(labels, "u");
    int indexYH2O = dataIndex(labels, "YH2O");
    int indexYH2 = dataIndex(labels, "YH2");
    int indexYH = dataIndex(labels, "YH");
    int indexrho = dataIndex(labels, "rho");
    
    int i(0);
    for(i=0; i<dataArray.size();i++)
    {
        if(indext>0)
            m_t.push_back(dataArray[i][indext]);
        
        if(indexT>0)
            m_T.push_back(dataArray[i][indexT]);
        
        if(indexYPAHnucl>0)
            m_YPAHnucl.push_back(dataArray[i][indexYPAHnucl]);
        
        if(indexYA1>0)
            m_YA1.push_back(dataArray[i][indexYA1]);
        
        if(indexYA2>0)
            m_YA2.push_back(dataArray[i][indexYA2]);
        
        if(indexYA3>0)
            m_YA3.push_back(dataArray[i][indexYA3]);
        
        if(indexYA4>0)
            m_YA4.push_back(dataArray[i][indexYA4]);
        
        if(indexYC2H2>0)
            m_YC2H2.push_back(dataArray[i][indexYC2H2]);
        
        if(indexYO2>0)
            m_YO2.push_back(dataArray[i][indexYO2]);
        
        if(indexYOH>0)
            m_YOH.push_back(dataArray[i][indexYOH]);
        
        if(indexx>0)
            m_x.push_back(dataArray[i][indexx]);
        
        if(indexu>0)
            m_u.push_back(dataArray[i][indexu]);
        
        if(indexYH2O>0)
            m_YH2O.push_back(dataArray[i][indexYH2O]);
        
        if(indexYH2>0)
            m_YH2.push_back(dataArray[i][indexYH2]);
        
        if(indexYH>0)
            m_YH.push_back(dataArray[i][indexYH]);
        
        if(indexrho>0)
            m_rho.push_back(dataArray[i][indexrho]);
        
        if(indexYA4imposed>0)
        {
            m_YA4imposed.push_back(dataArray[i][indexYA4imposed]);
        }
        
        if(indexwmA4>0)
        {
            m_wmA4.push_back(dataArray[i][indexwmA4]);
        }
        
    }
}



// returns the rank in m_t of the closest (superior) value to argument t

int HomogeneousGasPhase::getRankt(double t)
{
    double tCloser(0);
    int rankCloser(0);
    
    int rank(0);
    
    int size(0);
    int i(0);
    for(i=0; i<m_t.size();i++)
    {
        size++;
    }
    int rankMax = size - 1;
    
    double tmax = m_t[rankMax];
    double tmin = m_t[0];
    
    if(t>tmax)
    {
        //cout << endl;
        //cout << "t > tmax" << endl;
        tCloser = tmax;
        rankCloser = rankMax;
        
    }
    else if(t<tmin)
    {
        //cout << endl;
        //cout << "t < tmin" << endl;
        tCloser = tmin;
        rankCloser = 0;
    }
    else
    {
        
        while(m_t[rank] < t)
        {
            rank++;
        }
        
        rankCloser = rank;
    }
    
    return rankCloser;
}



double HomogeneousGasPhase::getValue(double t, vector<double> vector)
{
    int rank = getRankt(t);
    
    double tSup = m_t[rank];
    double coefSup(1.0);
    double coefInf(0.0);
    double interpolated(0);
    
    if(vector.size()>rank  &&  vector.size()>0)
    {
        if(t>=tSup || t<=m_t[0])
        {
            return vector[rank];
        }
        else  // we go from equations coefSup * m_t[rank] + coefInf * m_t[rank - 1] = t
            // and  coefInf + coefSup = 1
        {
            coefSup = (t - m_t[rank-1]) / (m_t[rank] - m_t[rank - 1]);
            coefInf = 1.0 - coefSup;
            interpolated = coefInf * vector[rank -1] + coefSup * vector[rank];
            return interpolated;
        }
    }
    else
    {
        cout << "insufficient information in vector of Homogeneous phase" << endl;
        return 0.0;
    }
}




double HomogeneousGasPhase::getT(double t)
{
    double value = getValue(t, m_T);
    return value;
}


double HomogeneousGasPhase::getYPAHnucl(double t)
{
    double value = getValue(t, m_YPAHnucl);
    return value;
}


double HomogeneousGasPhase::getYA1(double t)
{
    double value = getValue(t, m_YA1);
    return value;
}


double HomogeneousGasPhase::getYA2(double t)
{
    double value = getValue(t, m_YA2);
    return value;
}


double HomogeneousGasPhase::getYA3(double t)
{
    double value = getValue(t, m_YA3);
    return value;
}


double HomogeneousGasPhase::getYA4(double t)
{
    double value = getValue(t, m_YA4);
    return value;
}


double HomogeneousGasPhase::getYC2H2(double t)
{
    double value = getValue(t, m_YC2H2);
    return value;
}


double HomogeneousGasPhase::getYO2(double t)
{
    double value = getValue(t, m_YO2);
    return value;
}

double HomogeneousGasPhase::getYOH(double t)
{
    double value = getValue(t, m_YOH);
    return value;
}

double HomogeneousGasPhase::getu(double t)
{
    double value = getValue(t, m_u);
    return value;
}

double HomogeneousGasPhase::getYH2O(double t)
{
    double value = getValue(t, m_YH2O);
    return value;
}

double HomogeneousGasPhase::getYH2(double t)
{
    double value = getValue(t, m_YH2);
    return value;
}


double HomogeneousGasPhase::getYH(double t)
{
    double value = getValue(t, m_YH);
    return value;
}

double HomogeneousGasPhase::getrho(double t)
{
    double value = getValue(t, m_rho);
    return value;
}

double HomogeneousGasPhase::getx(double t)
{
    double value = getValue(t, m_x);
    return value;
}

double HomogeneousGasPhase::getYA4imposed(double t)
{
    double value = getValue(t, m_YA4imposed);
    return value;
}


double HomogeneousGasPhase::getwmA4(double t)
{
    double value = getValue(t, m_wmA4);
    return value;
}


double HomogeneousGasPhase::getConcentration(double Yspecies, double rho, double molarMass)
{
    // rho is initially in [kg/m3]. molarMass is in [g/mol] 
    double rhoGramm = rho * 1e3 *1e-6; // [in g/cm3]
    double Ni = avogadro * Yspecies * rhoGramm / molarMass;  // avogadro * [] * [mol/cm3] = [part/cm3]
    return Ni;  // [part/cm3]
}

double HomogeneousGasPhase::getConcentrationString(double Yspecies, double rho, string speciesName)
{
    double molarMass(1.0);
    if(speciesName == "A1")
    {
        molarMass = WA1;
    }
    else if(speciesName == "A2")
    {
        molarMass = WA2;
    }
    else if(speciesName == "A3")
    {
        molarMass = WA3;
    }
    else if (speciesName == "A4")
    {
        molarMass = WA4;
    }
    else if (speciesName == "C2H2")
    {
        molarMass = WC2H2;
    }
    else if (speciesName == "O2")
    {
        molarMass = WO2;
    }
    else if (speciesName == "OH")
    {
        molarMass = WOH;
    }
    else if (speciesName == "H2O")
    {
        molarMass = WH2O;
    }
    else if (speciesName == "H2")
    {
        molarMass = WH2;
    }
    else if (speciesName == "H")
    {
        molarMass = WH;
    }
    else
    {
        cout << "ERROR: missing molar mass" << endl;
    }
    
    double Ni;
    Ni = getConcentration(Yspecies, rho, molarMass);
    return Ni;  // in [part/cm3]
}


double HomogeneousGasPhase::getMolarConcentration(double t, double rho, string speciesName)
{
    double Yi(0.0);
    double molarMass(1.0);
    if(speciesName == "A1")
    {
        Yi = getYA1(t);
        molarMass = WA1;
    }
    else if(speciesName == "A2")
    {
        Yi = getYA2(t);
        molarMass = WA2;
    }
    else if(speciesName == "A3")
    {
        Yi = getYA3(t);
        molarMass = WA3;
    }
    else if (speciesName == "A4")
    {
        Yi = getYA4(t);
        molarMass = WA4;
    }
    else if (speciesName == "C2H2")
    {
        Yi = getYC2H2(t);
        molarMass = WC2H2;
    }
    else if (speciesName == "O2")
    {
        Yi = getYO2(t);
        molarMass = WO2;
    }
    else if (speciesName == "OH")
    {
        Yi = getYOH(t);
        molarMass = WOH;
    }
    else if (speciesName == "H2O")
    {
        Yi = getYH2O(t);
        molarMass = WH2O;
    }
    else if (speciesName == "H2")
    {
        Yi = getYH2(t);
        molarMass = WH2;
    }
    else if (speciesName == "H")
    {
        Yi = getYH(t);
        molarMass = WH;
    }
    else
    {
        cout << "ERROR: missing molar mass" << endl;
    }
    
    // rho is initially in [kg/m3]. molarMass is in [g/mol]
    double rhoGramm = rho * 1e3 *1e-6; // [in g/cm3]
    double ci = Yi * rhoGramm / molarMass;  // [] * [mol/cm3] = [mol/cm3]
    return ci;  // in [mol/cm3]
}


double HomogeneousGasPhase::getValueVectorwmA4(int i)
{
    return m_wmA4[i];
}

double HomogeneousGasPhase::getPperfectGas(double t)  // in Pa
{
    // constant perfect gases R1
    double R1 = 8.314;
    
    // air molar mass  28.965 g/mol
    double Mair = 28.965 * 1.0e-3; // kg/mol
    double R2 = R1/Mair;
    double rho =  getrho(t);
    double T = getT(t);
    
    // perfect gas law
    double P = rho * R2 * T;
    return P;  // in Pa
}
