//
//  Psd.cpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 29/08/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#include "Psd.hpp"

#include <iostream>
#include <string>
#include <vector>
#include <cmath>

using namespace std;


const double Psd::rhoSoot = 1.8; // [g/cm^3]
const double Psd::pi = 3.14159265;


Psd::Psd(Grid initialGrid, AerosolPhase* aeroSolPtr, double initNvT)
{
    std::vector<Bin> psd;
    double npInit = 0.0;
    double nvInit = 0.0;
    
    int i =0;
    for(i=0;i<initialGrid.getSize();i++)
    {
        GridCell gridCell = initialGrid.getGridCell(i);
        Bin bin = Bin(gridCell, npInit, nvInit);
        psd.push_back(bin);
    }
    
    m_binVector = psd;
    m_nvT = initNvT;
    m_aeroSolPtr = aeroSolPtr;
    m_smallestDetectedDiameter = 0.0; // default
}


Psd::Psd(Grid initialGrid, AerosolPhase* aeroSolPtr, double initNvT, double smallestDetectedDiameter)
{
    std::vector<Bin> psd;
    double npInit = 0.0;
    double nvInit = 0.0;
    
    int i =0;
    for(i=0;i<initialGrid.getSize();i++)
    {
        GridCell gridCell = initialGrid.getGridCell(i);
        Bin bin = Bin(gridCell, npInit, nvInit);
        psd.push_back(bin);
    }
    
    m_binVector = psd;
    m_nvT = initNvT;
    m_aeroSolPtr = aeroSolPtr;
    m_smallestDetectedDiameter = smallestDetectedDiameter; // default
}


Psd::Psd(Grid initialGrid, double n0, double x0, string ndType, string psdInitMethod)
{
    if(psdInitMethod == "sectional")
    {
        std::vector<Bin> psd;
        double initNvT(0.0);
        
        int i =0;
        for(i=0;i<initialGrid.getSize();i++)
        {
            GridCell gridCell = initialGrid.getGridCell(i);
            double mAvg = gridCell.getmAvg();
            double ndIniti = n0 / x0 * exp(- mAvg / x0);
            double nvIniti(0.0);
            
            if(ndType == "dm")
            {
                double mInf = gridCell.getmInf();
                double mSup = gridCell.getmSup();
                //nvIniti = ndIniti * (mSup - mInf); // simple approximation (uniform distribution inside the sections)
                double A = n0/x0;
                double B = - 1.0 / x0;
                double primitive_n_mSup = A / B * exp(B * mSup);
                double primitive_n_mInf = A / B * exp(B * mInf);
                nvIniti = primitive_n_mSup - primitive_n_mInf;
            }
            else if(ndType == "dlog")
            {
                double dInf = getdInfNano(i);
                double dSup = getdSupNano(i);
                double dlogInf = log10(dInf);
                double dlogSup = log10(dSup);
                nvIniti = ndIniti * (dlogSup - dlogInf);  // too simple approximation (uniform distribution inside the sections)
            }
            initNvT += nvIniti;
            
            Bin bin = Bin(gridCell, nvIniti);
            
            // modification for mAvgVar
            bin.setmAvgVarR(mAvg);
            
            psd.push_back(bin);
            
        }
        
        m_binVector = psd;
        m_nvT = initNvT;
        //cout << "initNvT = " << initNvT << endl;
        
        m_aeroSolPtr = nullptr;
        m_smallestDetectedDiameter = 0.0;
    }
    else
    {
        cout << "ERROR Psd creator pbeMethod" << endl;
    }
    
}


Psd::Psd(Grid initialGrid, double n0, double x0, string ndType, string psdInitMethod, double smallestDetectedDiameter)
{
    if(psdInitMethod == "sectional")
    {
        std::vector<Bin> psd;
        double initNvT(0.0);
        
        int i =0;
        for(i=0;i<initialGrid.getSize();i++)
        {
            GridCell gridCell = initialGrid.getGridCell(i);
            double mAvg = gridCell.getmAvg();
            double ndIniti = n0 / x0 * exp(- mAvg / x0);
            double nvIniti(0.0);
            
            if(ndType == "dm")
            {
                double mInf = gridCell.getmInf();
                double mSup = gridCell.getmSup();
                //nvIniti = ndIniti * (mSup - mInf); // simple approximation (uniform distribution inside the sections)
                double A = n0/x0;
                double B = - 1.0 / x0;
                double primitive_n_mSup = A / B * exp(B * mSup);
                double primitive_n_mInf = A / B * exp(B * mInf);
                nvIniti = primitive_n_mSup - primitive_n_mInf;
            }
            else if(ndType == "dlog")
            {
                double dInf = getdInfNano(i);
                double dSup = getdSupNano(i);
                double dlogInf = log10(dInf);
                double dlogSup = log10(dSup);
                nvIniti = ndIniti * (dlogSup - dlogInf);  // too simple approximation (uniform distribution inside the sections)
            }
            initNvT += nvIniti;
            
            Bin bin = Bin(gridCell, nvIniti);
            
            // modification for mAvgVar
            bin.setmAvgVarR(mAvg);
            
            psd.push_back(bin);
            
        }
        
        m_binVector = psd;
        m_nvT = initNvT;
        //cout << "initNvT = " << initNvT << endl;
        
        m_aeroSolPtr = nullptr;
        m_smallestDetectedDiameter = smallestDetectedDiameter;
    }
    else
    {
        cout << "ERROR Psd creator pbeMethod" << endl;
    }
    
}



Psd::Psd(Grid initialGrid, int bin, double NvTinit, string PsdInitMethod,  double smallestDetectedDiameter)
{
    if(PsdInitMethod == "sectional")
    {
        std::vector<Bin> psd;
        
        int i =0;
        for(i=0;i<initialGrid.getSize();i++)
        {
            GridCell gridCell = initialGrid.getGridCell(i);
            double nvIniti(0.0);
            if(i==bin)
            {
                nvIniti = NvTinit;
            }
            else
            {
                nvIniti = 0.0;
            }
            Bin bin = Bin(gridCell, nvIniti);
            psd.push_back(bin);
            
        }
        
        m_binVector = psd;
        m_nvT = NvTinit;
        
        m_aeroSolPtr = nullptr;
        m_smallestDetectedDiameter = smallestDetectedDiameter;
    }
    else
    {
        cout << "ERROR Psd creator pbeMethod" << endl;
    }
    
}


Psd::Psd(Grid initialGrid, int bin, double NvTinit, string ndType, string PsdInitMethod)
{
    if(PsdInitMethod == "sectional")
    {
        std::vector<Bin> psd;
        
        int i =0;
        for(i=0;i<initialGrid.getSize();i++)
        {
            GridCell gridCell = initialGrid.getGridCell(i);
            double nvIniti(0.0);
            if(i==bin)
            {
                nvIniti = NvTinit;
            }
            else
            {
                nvIniti = 0.0;
            }
            Bin bin = Bin(gridCell, nvIniti);
            psd.push_back(bin);
            
        }
        
        m_binVector = psd;
        m_nvT = NvTinit;
        
        m_aeroSolPtr = nullptr;
        m_smallestDetectedDiameter = 0.0;
    }
    else
    {
        cout << "ERROR Psd creator pbeMethod" << endl;
    }
    
}


Psd::Psd(Grid initialGrid, AerosolPhase* aeroSolPtr, double n0, double x0, string ndType, string psdInitMethod)
{
    if(psdInitMethod == "npResidual" || psdInitMethod == "npResidualPart" || psdInitMethod == "hybrid" || psdInitMethod == "hybridPart")
    {
        m_aeroSolPtr = aeroSolPtr;
        
        std::vector<Bin> psd;
        double initNvT(0.0);
        vector<double> nvInitiVec;
        int NpTot = aeroSolPtr->getNpTot();
        
        // first loop in order to calculate exact nvT and nvIniti vector. also creates a BinVector and sets npi in each Bin
        int i =0;
        for(i=0;i<initialGrid.getSize();i++)
        {
            GridCell gridCell = initialGrid.getGridCell(i);
            double mAvg = gridCell.getmAvg();
            double ndIniti = n0 / x0 * exp(- mAvg / x0);
            double nvIniti(0.0);
            
            double mInf = gridCell.getmInf();
            double mSup = gridCell.getmSup();
            
            if(ndType == "dm")
            {
                //nvIniti = ndIniti * (mSup - mInf); // too simple approximation (uniform distribution inside the sections)
                double A = n0/x0;
                double B = - 1.0 / x0;
                double primitive_n_mSup = A / B * exp(B * mSup);
                double primitive_n_mInf = A / B * exp(B * mInf);
                nvIniti = primitive_n_mSup - primitive_n_mInf;
            }
            else if(ndType == "dlog")
            {
                double dInf = getdInfNano(i);
                double dSup = getdSupNano(i);
                double dlogInf = log10(dInf);
                double dlogSup = log10(dSup);
                nvIniti = ndIniti * (dlogSup - dlogInf);  // too simple approximation (uniform distribution inside the sections)
            }
            
            initNvT += nvIniti;
            nvInitiVec.push_back(nvIniti);
            
            //int npl = aeroSolPtr->countNpl(mInf, mSup);
            // substitute to count in aeroSolPhase. now count directly in Psd
            int j(0);
            int npl(0);
            double mSumBin(0.0);
            double mAvgVarStoch(0.0);
            
            for(j=0; j<NpTot; j++)
            {
                double m = aeroSolPtr->getMass(j);
                if(m >= mInf  &&  m < mSup)
                {
                    npl++;
                    mSumBin += m;
                }
                
            }
            if (npl>0)
            {
                mAvgVarStoch = mSumBin / npl;
            }
            else
            {
                mAvgVarStoch = mAvg;
            }
            
            //
            
            Bin bin = Bin(gridCell, npl);
            bin.setmAvgVarStoch(mAvgVarStoch);
            // for Psd creation, mAvgVarR set at mAvgConst (or should be at mAvgVarStoch?). Then mAvgVar calculated from mAvgVarStoch, NvStoch, mAvgR, NvR
            //bin.setmAvgVarR(mAvg);
            bin.setmAvgVarR(mAvgVarStoch);
            
            psd.push_back(bin);
            
        }
        
        m_nvT = initNvT;
        
        
        // second loop to calculate and set nvStoch and set nvResidual
        i =0;
        for(i=0;i<initialGrid.getSize();i++)
        {
            int npi = psd[i].getNp();
            
            double nvStoch = double(npi) /double(NpTot) * m_nvT;
            double nvResidual = nvInitiVec[i] - nvStoch;
            
            //cout << "Psd::Psd  nvInitiVec = " << nvInitiVec[i] << " nvStoch = " << nvStoch << "   nvResidual = " << nvResidual << "   npi = " << npi << "   NvT = " << getNvT() << endl;
            
            psd[i].setNvResidual(nvResidual);
            psd[i].setNvStochastic(nvStoch);
            
            // for Psd creation, mAvgVarR set at mAvgConst. Then mAvgVar calculated from mAvgVarStoch, NvStoch, mAvgR, NvR
            double mAvgVarStoch = psd[i].getmAvgVarStoch();
            double mAvgVarR = psd[i].getmAvgVarR();
            double mAvgVarGlobal(0.0);
            
            if((nvStoch + nvResidual) == 0.0)
            {
                double mAvg = psd[i].getmAvg();
                mAvgVarGlobal = mAvg;
            }
            else
            {
                GridCell gridCell0 = initialGrid.getGridCell(0);
                double mNucleation = gridCell0.getmAvg();
                mAvgVarGlobal = calcmAvgVar(nvStoch, mAvgVarStoch, nvResidual, mAvgVarR, i, mNucleation);
            }
            
            //setmAvgVar(i, mAvgVarGlobal);
            
            if(mAvgVarGlobal > 0)
            {
                psd[i].setmAvgVar(mAvgVarGlobal);
            }
            else
            {
                cout << "calculated mAvgVar <= 0" << endl;
            }
            
        }
        
        m_binVector = psd;
        
        
        
        m_smallestDetectedDiameter = 0.0;
    }
    else
    {
        cout << "ERROR Psd creator pbeMethod" << endl;
    }
    
}



Psd::Psd(Grid initialGrid, AerosolPhase* aeroSolPtr, double n0, double x0, string ndType, string psdInitMethod, double smallestDetectedDiameter)
{
    if(psdInitMethod == "npResidual" || psdInitMethod == "npResidualPart" || psdInitMethod == "hybrid" || psdInitMethod == "hybridPart")
    {
        m_aeroSolPtr = aeroSolPtr;
        
        std::vector<Bin> psd;
        double initNvT(0.0);
        vector<double> nvInitiVec;
        int NpTot = aeroSolPtr->getNpTot();
        
        // first loop in order to calculate exact nvT and nvIniti vector. also creates a BinVector and sets npi in each Bin
        int i =0;
        for(i=0;i<initialGrid.getSize();i++)
        {
            GridCell gridCell = initialGrid.getGridCell(i);
            double mAvg = gridCell.getmAvg();
            double ndIniti = n0 / x0 * exp(- mAvg / x0);
            double nvIniti(0.0);
            
            double mInf = gridCell.getmInf();
            double mSup = gridCell.getmSup();
            
            if(ndType == "dm")
            {
                //nvIniti = ndIniti * (mSup - mInf); // too simple approximation (uniform distribution inside the sections)
                double A = n0/x0;
                double B = - 1.0 / x0;
                double primitive_n_mSup = A / B * exp(B * mSup);
                double primitive_n_mInf = A / B * exp(B * mInf);
                nvIniti = primitive_n_mSup - primitive_n_mInf;
            }
            else if(ndType == "dlog")
            {
                double dInf = getdInfNano(i);
                double dSup = getdSupNano(i);
                double dlogInf = log10(dInf);
                double dlogSup = log10(dSup);
                nvIniti = ndIniti * (dlogSup - dlogInf);  // too simple approximation (uniform distribution inside the sections)
            }
            
            initNvT += nvIniti;
            nvInitiVec.push_back(nvIniti);
            
            //int npl = aeroSolPtr->countNpl(mInf, mSup);
            // substitute to count in aeroSolPhase. now count directly in Psd
            int j(0);
            int npl(0);
            double mSumBin(0.0);
            double mAvgVarStoch(0.0);
            
            for(j=0; j<NpTot; j++)
            {
                double m = aeroSolPtr->getMass(j);
                if(m >= mInf  &&  m < mSup)
                {
                    npl++;
                    mSumBin += m;
                }
                
            }
            if (npl>0)
            {
                mAvgVarStoch = mSumBin / npl;
            }
            else
            {
                mAvgVarStoch = mAvg;
            }
            
            //
            
            Bin bin = Bin(gridCell, npl);
            bin.setmAvgVarStoch(mAvgVarStoch);
            // for Psd creation, mAvgVarR set at mAvgConst (or should be at mAvgVarStoch?). Then mAvgVar calculated from mAvgVarStoch, NvStoch, mAvgR, NvR
            //bin.setmAvgVarR(mAvg);
            bin.setmAvgVarR(mAvgVarStoch);
            
            psd.push_back(bin);
            
        }
        
        m_nvT = initNvT;
        
        
        // second loop to calculate and set nvStoch and set nvResidual
        i =0;
        for(i=0;i<initialGrid.getSize();i++)
        {
            int npi = psd[i].getNp();
            
            double nvStoch = double(npi) /double(NpTot) * m_nvT;
            double nvResidual = nvInitiVec[i] - nvStoch;
            
            //cout << "Psd::Psd  nvInitiVec = " << nvInitiVec[i] << " nvStoch = " << nvStoch << "   nvResidual = " << nvResidual << "   npi = " << npi << "   NvT = " << getNvT() << endl;
            
            psd[i].setNvResidual(nvResidual);
            psd[i].setNvStochastic(nvStoch);
            
            // for Psd creation, mAvgVarR set at mAvgConst. Then mAvgVar calculated from mAvgVarStoch, NvStoch, mAvgR, NvR
            double mAvgVarStoch = psd[i].getmAvgVarStoch();
            double mAvgVarR = psd[i].getmAvgVarR();
            double mAvgVarGlobal(0.0);
            
            if((nvStoch + nvResidual) == 0.0)
            {
                double mAvg = psd[i].getmAvg();
                mAvgVarGlobal = mAvg;
            }
            else
            {
                GridCell gridCell0 = initialGrid.getGridCell(0);
                double mNucleation = gridCell0.getmAvg();
                mAvgVarGlobal = calcmAvgVar(nvStoch, mAvgVarStoch, nvResidual, mAvgVarR, i, mNucleation);
            }
            
            //setmAvgVar(i, mAvgVarGlobal);
            
            if(mAvgVarGlobal > 0)
            {
                psd[i].setmAvgVar(mAvgVarGlobal);
            }
            else
            {
                cout << "calculated mAvgVar <= 0" << endl;
            }
            
        }
        
        m_binVector = psd;
        
        
        
        m_smallestDetectedDiameter = smallestDetectedDiameter;
    }
    else
    {
        cout << "ERROR Psd creator pbeMethod" << endl;
    }
    
}




int Psd::getSize()
{
    int size(0);
    int i(0);
    for(i=0; i<m_binVector.size();i++)
    {
        size++;
    }
    return size;
}


double Psd::getNvT()
{
    return m_nvT;
}

double Psd::getSumNv()
{
    int i(0);
    double sumNv(0.0);
    int nBins = getSize();
    for(i=0; i<nBins; i++)
    {
        double Nvi = getNv(i);
        sumNv += Nvi;
    }
    return sumNv;
}

double Psd::getmTot()
{
    int i(0);
    double sumMass(0.0);
    int nBins = getSize();
    for(i=0; i<nBins; i++)
    {
        double Nvi = getNv(i);
        //double mAvgi = getmAvg(i);
        double mAvgVari = getmAvgVar(i);
        
        //sumMass += Nvi*mAvgi;
        sumMass += Nvi*mAvgVari;
    }
    return sumMass;
}


Bin Psd::getBin(int i)
{
    return m_binVector[i];
}


Bin* Psd::getBinPtr(int i)
{
    Bin* ptr = &m_binVector[i];
    return ptr;
}


int Psd::getNpT()
{
    int npT(0);
    
    int i(0);
    for(i=0; i<m_binVector.size();i++)
    {
        npT+= getNp(i);
    }
    
    return npT;
}


double Psd::getmAvg(int i)
{
    Bin* ptr = getBinPtr(i);
    return ptr->getmAvg();
}

double Psd::getdAvgNano(int i)
{
    Bin* ptr = getBinPtr(i);
    return ptr->getdAvgNano();
}

double Psd::getsAvg(int i)
{
    //Bin* ptr = getBinPtr(i);
    //return ptr->getsAvg();
    double mass = getmAvgVar(i);
    double s = getSfromMass(mass);
    return s;
}


double Psd::getmInf(int i)
{
    Bin* ptr = getBinPtr(i);
    return ptr->getmInf();
}

double Psd::getdInfNano(int i)
{
    Bin* ptr = getBinPtr(i);
    return ptr->getdInfNano();
}

double Psd::getsInf(int i)
{
    Bin* ptr = getBinPtr(i);
    return ptr->getsInf();
}

double Psd::getmSup(int i)
{
    Bin* ptr = getBinPtr(i);
    return ptr->getmSup();
}

double Psd::getdSupNano(int i)
{
    Bin* ptr = getBinPtr(i);
    return ptr->getdSupNano();
}

double Psd::getsSup(int i)
{
    Bin* ptr = getBinPtr(i);
    return ptr->getsSup();
}

int Psd::getNp(int i)
{
    Bin* ptr = getBinPtr(i);
    return ptr->getNp();
}

double Psd::getNv(int i)
{
    Bin* ptr = getBinPtr(i);
    return ptr->getNv();
}


double Psd::calculateNm(int i)
{
    double mInf = getmInf(i);
    double mSup = getmSup(i);
    double nv = getNv(i);
    double nm = nv / (mSup - mInf);
    
    return nm;
}

double Psd::calculateNd(int i)
{
    double dInf = getdInfNano(i);
    double dSup = getdSupNano(i);
    double dlogInf = log10(dInf);
    double dlogSup = log10(dSup);
    double nv = getNv(i);
    double nd = nv / (dlogSup - dlogInf);
    
    return nd;
}

double Psd::calculateNmStochastic(int i)
{
    Bin* binPtr = getBinPtr(i);
    double mInf = getmInf(i);
    double mSup = getmSup(i);
    double nv = binPtr->getNv();
    double nvResidual = binPtr->getNvResidual();
    double nvStochastic = nv - nvResidual;
    double nmStochastic = nvStochastic / (mSup - mInf);
    
    return nmStochastic;
}


double Psd::calculateMv(int i)
{
    double mAvg = getmAvg(i);
    double nv = getNv(i);
    double mv = mAvg * nv;
    return mv;
}

double Psd::calculateMvT()
{
    double mvT(0);
    int i(0);
    for(i=0; i<m_binVector.size(); i++)
    {
        mvT+= calculateMv(i);
    }
    return mvT;
}


void Psd::setNvt(double nvt)
{
    m_nvT = nvt;
}



void Psd::calcAndSetAllNv()
{
    int npT = getNpT();
    int np(0);
    double nvStochastic(0.0);
    
    int i(0);
    for(i=0;i<m_binVector.size();i++)
    {
        np = getNp(i);
        nvStochastic = double(np) / double(npT) * m_nvT;
        Bin* ptr = getBinPtr(i);
        ptr->setNvStochastic(nvStochastic);
    }
}



void Psd::countAndSetAllNp()
{
    double mInf(0);
    double mSup(0);
    int NpTot = m_aeroSolPtr->getNpTot();
    
    int i(0);
    for(i=0; i<m_binVector.size();i++)
    {
        Bin* ptr = getBinPtr(i);
        mInf = ptr->getmInf();
        mSup = ptr->getmSup();
        
        int npl(0);
        double mAvgVarStoch(0.0);
        double mSumBin(0.0);
        
        // inserted substitute for AerosolPhase::countNpl. Count and mean now handled directly in Psd::
        int j(0);
        for(j=0; j<NpTot; j++)
        {
            double m = m_aeroSolPtr->getMass(j);
            if(m >= mInf  &&  m < mSup)
            {
                npl++;
                mSumBin += m;
                
                //for print
                if(i>0)
                {
                    //cout << "section: " << i << "   m = " << m << endl;
                }
            }
        }
        if (i > 0 & npl > 0)
        {
            //cout << "section: " << i << "   np = " << npl << endl;
        }
        
        if (npl>0)
        {
            mAvgVarStoch = mSumBin / npl;
        }
        else
        {
            mAvgVarStoch = ptr->getmAvg();
        }
        //
        
        //npl = m_aeroSolPtr->countNpl(mInf, mSup); was done this way before. Now count directly in Psd::countAndSetAllNp
        ptr->setNp(npl);
        ptr->setmAvgVarStoch(mAvgVarStoch);
        
    }
}



void Psd::showPsd()
{
    cout << endl << endl;
    cout << "nvT = " << m_nvT << "   npTot = " << getNpT() << endl;
    cout << "mAvg       np  nv       nvStoch     nvResidual      d(nm)         nd         nm   mAvgVar"<<endl;
    int i(0);
    for(i=0; i<m_binVector.size();i++)
    {
        Bin* ptr = getBinPtr(i);
        double nv = getNv(i);
        double nvResidual = ptr->getNvResidual();
        double nvStochastic = nv - nvResidual;
        
        cout << getmAvg(i) << "   " << getNp(i) << "   " << nv << "   "<<nvStochastic << "   " << nvResidual << "   " << getdAvgNano(i) << "   " << calculateNd(i) << "   " << calculateNm(i) << "   " << ptr->getmAvgVar() <<endl;
    }
    cout << endl << endl;
}


double Psd::calculateNvTotDetected()
{
    double NvTotDetected(0.0);
    int nbBins = getSize();
    int i(0);
    for(i=0;i<nbBins;i++)
    {
        double dAvgi = getdAvgNano(i);
        if(dAvgi>=m_smallestDetectedDiameter)
        {
            NvTotDetected+= getNv(i);
        }
    }
    return NvTotDetected;
}


double Psd::getSmallestDetectedDiameter()
{
    return m_smallestDetectedDiameter;
}


double Psd::getmAvgVar(int i)
{
    Bin* ptr = getBinPtr(i);
    return ptr->getmAvgVar();
}

void Psd::setmAvgVar(int i, double mAvgVar)
{
    Bin* ptr = getBinPtr(i);
    return ptr->setmAvgVar(mAvgVar);
}

void Psd::setmAvgVar_as_mAvgConst()
{
    int nBins = getSize();
    int i(0);
    for(i=0; i<nBins; i++)
    {
        double mAvgConst = getmAvg(i);
        setmAvgVar(i, mAvgConst);
    }
}

void Psd::setmAvgVarR_as_mAvgConst()
{
    int nBins = getSize();
    int i(0);
    for(i=0; i<nBins; i++)
    {
        double mAvgConst = getmAvg(i);
        Bin* binPtr = getBinPtr(i);
        binPtr->setmAvgVarR(mAvgConst);
    }
}

void Psd::setmAvgVar_as_mAvgVarStoch()
{
    int nBins = getSize();
    int i(0);
    for(i=0; i<nBins; i++)
    {
        double mAvgVarStoch = getmAvgVarStoch(i);
        if (mAvgVarStoch > 0)
        {
            Bin* binPtr = getBinPtr(i);
            binPtr->setmAvgVar(mAvgVarStoch);
        }
    }
}

void Psd::setmAvgVarR_as_mAvgVarStoch()
{
    int nBins = getSize();
    int i(0);
    for(i=0; i<nBins; i++)
    {
        double mAvgVarStoch = getmAvgVarStoch(i);
        Bin* binPtr = getBinPtr(i);
        binPtr->setmAvgVarR(mAvgVarStoch);
    }
}

double Psd::calcmAvgVar(double NvStoch, double mAvgVarStoch, double NvR, double mAvgVarR, int bin, double mNucleation)
{
    double mAvgVar(0.0);
    //if((NvStoch + NvR) == 0.0)
    if((NvStoch + NvR) <= m_nvT / 1.0e20 )
    {
        mAvgVar = mAvgVarStoch; // mAvgConst if Nv = 0 in the bin. Can happen at Psd initialization so that we cannot get mAvg through Psd object
        //mAvgVar = getmAvg(bin); // mAvgConst if Nv = 0 in the bin
        //cout << "mAvgVar = " << mAvgVar << endl;
    }
    else
    {
        if(bin == 0) // different for bin = 0 because nucleation must happen only at mAvgConst. Otherwise, mass addition by nucleation is not controled and there is an error on global mass (if new stoch particles are allocated at mAvgVar(i = 0) differente from mAvgConst (i = 0).
            // Maybe maintain uniform reallocation for bin 0 but calculate mAvgVar as for oter sections
        {
            //mAvgVar = mNucleation;
            mAvgVar = (NvStoch * mAvgVarStoch + NvR * mAvgVarR) / (NvStoch + NvR);
        } else {
            mAvgVar = (NvStoch * mAvgVarStoch + NvR * mAvgVarR) / (NvStoch + NvR);
        }
        //cout << "Psd::calcmAvgVar mAvgVar = " << mAvgVar << "   NvStoch = " << NvStoch << "   NvR = " << NvR << "   mAvgVarStoch = " << mAvgVarStoch << "   mAvgVarR = " << mAvgVarR << endl << endl;
    }
    
    if (mAvgVar <= 0.0)
    {
        cout << "mAvgVar of bin " << bin << " <= 0" << endl;
        cout << "NvStoch = " << NvStoch << "   mAvgVarStoch = " << mAvgVarStoch << "   NvR = " << NvR << "   mAvgVarR = " << mAvgVarR << endl;
    }
    return mAvgVar;
}


double Psd::getmAvgVarR(int i)
{
    Bin* ptr = getBinPtr(i);
    return ptr->getmAvgVarR();
}

double Psd::getmAvgVarStoch(int i)
{
    Bin* ptr = getBinPtr(i);
    return ptr->getmAvgVarStoch();
}

void Psd::calcAndSetAllmAvgVar()
{
    int nBins = getSize();
    int NpTot = getNpT();
    int i(0);
    for(i=0; i< nBins; i++)
    {
        Bin* binPtr = getBinPtr(i);
        int npl = getNp(i);
        double NvStoch = double(npl)/double(NpTot) * getNvT();
        double NvR = binPtr->getNvResidual();
        double mAvgVarR = binPtr->getmAvgVarR();
        double mAvgVarStoch = binPtr->getmAvgVarStoch();
        
        double mAvgVarGlobal(0.0);
        if((NvStoch + NvR) == 0.0)
        {
            mAvgVarGlobal = binPtr->getmAvg();
        }
        else
        {
            double mNucleation = getmAvg(0);
            mAvgVarGlobal = calcmAvgVar(NvStoch, mAvgVarStoch, NvR, mAvgVarR, i, mNucleation);
        }
        if (mAvgVarGlobal > 0)
        {
            binPtr->setmAvgVar(mAvgVarGlobal);
        }
        else
        {
            cout << "calculated mAvgVar <= 0" << endl;
        }
        
    }
}


double Psd::getSfromMass(double m)  // for sphere
{
    double v = m/rhoSoot;  // in [cm3]    m_mAvg in [g]
    double oneThird = 1.0/3.0;
    double d3 = v*6.0/pi;
    double dcm = pow(d3, oneThird);  // [in cm]
    double surf = pi * pow(dcm, 2.0); // [in cm2]
    return surf;
    
}
