//
//  SootSourceTerm.cpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 04/09/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#include "SootSourceTerm.hpp"

#include <iostream>
#include <string>
#include <vector>
#include <cmath>

using namespace std;

//constant values used for source terms calculation
const double SootSourceTerm::avogadro = 6.024e23;
const double SootSourceTerm::cmass = 12.0 * 1.6605e-24;  // [g]
const double SootSourceTerm::boltzmann = 1.3807e-16;
const double SootSourceTerm::rhoSoot = 1.8; // [g/cm^3]
const double SootSourceTerm::pi = 3.14159265;
const double SootSourceTerm::epsilon = 2.2;
//const double SootSourceTerm::epsilonCond = 1.3;  // from from KIT Mauss, Netzell p465
const double SootSourceTerm::epsilonCond = 2.2;  // from Frenklach_Wang
const double SootSourceTerm::chi = 2.3e15; // [active sites/cm2] from APPEL, BOCKHORN, FRENKLACH, 2000: Combust. Flame 121:122 (2000).
const double SootSourceTerm::Rconst = 1.987e-3;    //  ! R in kcal/mole

// surface growth mechanism from APPEL, BOCKHORN, FRENKLACH, 2000: Combust. Flame 121:122 (2000).
const double SootSourceTerm::fR1_A = 4.17e13;
const double SootSourceTerm::fR1_n = 0.0;
const double SootSourceTerm::fR1_E = 13.0;
const double SootSourceTerm::rR1_A = 3.9e12;
const double SootSourceTerm::rR1_n = 0.0;
const double SootSourceTerm::rR1_E = 11.0;
const double SootSourceTerm::fR2_A = 1.0e10;
const double SootSourceTerm::fR2_n = 0.734;
const double SootSourceTerm::fR2_E = 1.43;
const double SootSourceTerm::rR2_A = 3.68e08;
const double SootSourceTerm::rR2_n = 1.139;
const double SootSourceTerm::rR2_E = 17.1;
const double SootSourceTerm::fR3_A = 2.0e13;
const double SootSourceTerm::fR3_n = 0.0;
const double SootSourceTerm::fR3_E = 0.0;
const double SootSourceTerm::fR4_A = 8.0e7;
const double SootSourceTerm::fR4_n = 1.56;
const double SootSourceTerm::fR4_E = 3.8;
const double SootSourceTerm::fR5_A = 2.2e12;
const double SootSourceTerm::fR5_n = 0.0;
const double SootSourceTerm::fR5_E = 7.5;
const double SootSourceTerm::fR6_A = 0.13;
const double SootSourceTerm::fR6_n = 0.0;
const double SootSourceTerm::fR6_E = 0.0;

//const double C1 = 1.4558e-3; // conversion to g [ g.m^-1.s^-1.K^(-1/2) ]
//const double C2 = 110.4;   // [K]


SootSourceTerm::SootSourceTerm(HomogeneousGasPhase* gasPhasePtr, Psd* psdPtr, string solverType) : m_gasPhasePtr(gasPhasePtr), m_psdPtr(psdPtr), m_solverType(solverType), m_aggloType("standard"), m_nuclType("standard"), m_hacaType("standard"), m_condType("standard"), m_oxi02Type("standard"), m_oxiOHType("standard"), m_betaConst(0.0), m_consoTotPAH(0.0), m_dotmPAHNucl(0.0), m_alphaHACA(0.27), m_PAHType("nofeedback")
{
    double YA4init = gasPhasePtr->getYA4(0.0);
    double rhoGramm = gasPhasePtr->getrho(0.0) / 1000.0;
    m_CmA4corrected = YA4init * rhoGramm;   // [g/cm3]
}


// without pointer at HomogeneousGasPhase for analytical cases

SootSourceTerm::SootSourceTerm(Psd* psdPtr, string solverType) : m_psdPtr(psdPtr), m_solverType(solverType), m_aggloType("standard"), m_nuclType("standard"), m_hacaType("standard"), m_condType("standard"), m_oxi02Type("standard"), m_oxiOHType("standard"), m_betaConst(0.0), m_consoTotPAH(0.0), m_dotmPAHNucl(0.0), m_alphaHACA(0.27), m_PAHType("nofeedback")
{
    
}


double SootSourceTerm::partDimerSource(double t)  // en fait ici on devrait calculer dotDimer. Puis répartir entre nucleation dotH et condensation dotCond par QSS. Pour l'instant 100% va en nucléation
{
    double dotDimer(0);
    
    if(m_solverType == "analytical")
    {
        dotDimer = 0;
    }
    
    else if(m_solverType == "HomogeneousGasPhase")
    {
        double nuclMassSource(0);
        double T = m_gasPhasePtr->getT(t);
        
        double YPAH(0.0);
        
        if(m_PAHType == "nofeedback")
        {
            YPAH = m_gasPhasePtr->getYA4(t);  // no consumption of pyrene in gas phase
        }
        else if(m_PAHType == "corrected")
        {
            YPAH = YA4corrected(t);     // approximation of YA4 - YA4 consumed
        }
        else if(m_PAHType == "hybrid")
        {
            YPAH = YA4hybrid(t);        // YA4 imposed for the range of data available (from paper's graph and corrected for smaller x)
        }
        else
        {
            cout << "ERROR m_PAHType" << endl;
            YPAH = 0.0;
        }
        
        double rho = m_gasPhasePtr->getrho(t);
        double NPAH = m_gasPhasePtr->getConcentrationString(YPAH, rho, "A4");  // in [part/cm3]
        
        if(m_nuclType == "standard")
        {
            double mPAH = 16.0 * cmass;   // [g]  pyrene. Only C atoms mass considered as a simplification.
            double mPrimaryParticle = 2.0 * mPAH;  // [g]
            double beta = betaNucl(mPAH, T);
            
            double gammaStandard = 1.0; // 1.0 from Appel and MOM Berkeley. 0.025 from KIT Blanquart, Pitsch, p476, table 1. gamma for pyrene-A4
            
            nuclMassSource = gammaStandard * 2 * mPAH * beta * pow(NPAH,2.0); // [g/s/cm^3]
            m_dotmPAHNucl = nuclMassSource;
            
            dotDimer = nuclMassSource / mPrimaryParticle ;  // [part/s/cm^3]
            
        }
        else if (m_nuclType == "gammaNucl")
        {
            double mPAH = 16.0 * cmass;   // pyrene. Only C atoms mass considered as a simplification.
            double mPrimaryParticle = 2.0 * mPAH;
            double beta = betaNucl(mPAH, T);
            
            nuclMassSource = m_gammaNucl * 2 * mPAH * beta * pow(NPAH,2.0);
            m_dotmPAHNucl = nuclMassSource;
            
            dotDimer = nuclMassSource / mPrimaryParticle ;  // [part/s/cm^3]
        }
        
        else if(m_nuclType == "kraft")
        {
            dotDimer = 0.0;  //
            
        }
        else if(m_nuclType == "dotHconst")
        {
            dotDimer = 0.0;
        }
        else
        {
            
            dotDimer = 0.0;
            cout << "error m_nuclType" << endl;
            m_dotmPAHNucl = 0.0;
        }
    }
    else
    {
        cout << "ERROR: solverType" << endl;
    }
    
    return dotDimer;
}


double SootSourceTerm::partNuclSource(double t)
{
    double dotH(0.0);
    
    if(m_solverType == "analytical")
    {
        if(m_nuclType == "dotHconst")
        {
            dotH = m_b0;
        }
        else if(m_nuclType == "exponential")
        {
            dotH = 0.0;  // exponential nucleation at all sizes included added to partAggloSourceVector
        }
        else
        {
            dotH = 0.0;
            cout << "ERROR nuclType" << endl;
        }
    }
    
    else if(m_solverType == "HomogeneousGasPhase")
    {
        double T = m_gasPhasePtr->getT(t);
        if(m_nuclType=="kraft")
        {
            double mPAH = 16.0 * cmass;   // [g]
            double YPAH(0.0);
            
            if(m_PAHType == "nofeedback")
            {
                YPAH = m_gasPhasePtr->getYA4(t);  // no consumption of pyrene in gas phase
            }
            else if(m_PAHType == "corrected")
            {
                YPAH = YA4corrected(t);     // approximation of YA4 - YA4 consumed
            }
            else if(m_PAHType == "hybrid")
            {
                YPAH = YA4hybrid(t);        // YA4 imposed for the range of data available (from paper's graph and corrected for smaller x)
            }
            else
            {
                cout << "ERROR m_PAHType" << endl;
                YPAH = 0.0;
            }
            
            double rho = m_gasPhasePtr->getrho(t);
            double rhoGramm = rho * 1e3 *1e-6; // [in g/cm3]
            double molarMass = 16.0 * cmass * avogadro;
            double cPAH = YPAH * rhoGramm / molarMass;  // [mol/cm3]
            double NPAH = cPAH *avogadro;    // [part/cm3]
            
            double beta = betaNucl(mPAH, T);
            dotH = 0.5 * beta * NPAH * NPAH;
            
            
            double partPAHConsumed = 2*dotH;  // [part/cm3/s]
            m_dotmPAHNucl = partPAHConsumed * mPAH;   // in [g/cm3/s]
        }
        else if(m_nuclType=="standard")
        {
            double dimerC = dimerConcentration(t);   // [part/cm3]
            
            double mDimer = 32.0 * cmass;  // case of dimers formed of 2 pyrene molecules
            
            dotH = 0.5 * betaNucl(mDimer, T) * pow(dimerC, 2.0) ;
            
        }
        else
        {
            dotH = 0.0;
            cout << "ERROR nuclType" << endl;
        }
    }
    else
    {
        dotH = 0.0;
        cout << "ERROR solverType" << endl;
    }
    
    return dotH;   // [part/cm3/s]   now nuclei mass = 64 cmass
}


vector<double> SootSourceTerm::partAggloSourceVector(double t)
{
    vector<double> dotAlVector;
    
    if(m_solverType=="analytical")
    {
        double dotAl(0);
        int j(0);
        
        if(m_massTransferType == "geo2SPAMM")
        {
            for(j=0;j<m_psdPtr->getSize();j++)
            {
                dotAl = 0;
                cout << "ERROR: solverType analytical not coded for massTransferType == geo2SPAMM" << endl;
                //dotAl = wmTotj(j, T, P) / m_psdPtr->getmAvg(j); // [part/s/cm3] = [g/s/cm3] / [g/part]
                dotAlVector.push_back(dotAl);  // [part/s/cm^3]
            }
        }
        else if(m_massTransferType == "aggloTensor")
        {
            for(j=0;j<m_psdPtr->getSize();j++)
            {
                dotAl = tensorWmTotk(j) / m_psdPtr->getmAvg(j);  // [part/s/cm3] = [g/s/cm3] / [g/part]. here j is the destination bin
                dotAlVector.push_back(dotAl);  // [part/s/cm^3]
            }
        }
        else if(m_massTransferType == "kumar96")
        {
            for(j=0;j<m_psdPtr->getSize();j++)
            {
                dotAl = kumarAggloSource(j);
                dotAlVector.push_back(dotAl);  // [part/s/cm^3]
            }
            
        }
        
        //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        // just in case m_nuclType == exponential
        // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        if(m_nuclType=="exponential")
        {
            vector<double> dotBlVector = testCaseNuclSourceVector();
            j=0;
            for(j=0;j<dotAlVector.size();j++)
            {
                dotAlVector[j] += dotBlVector[j];
            }
        }
        //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        // just in case m_nuclType == exponential
        // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    }
    else if(m_solverType == "HomogeneousGasPhase")
    {
        double T = m_gasPhasePtr->getT(t);
        double P = m_gasPhasePtr->getPperfectGas(t);
        
        double dotAl(0);
        int j(0);
        
        if(m_massTransferType == "geo2SPAMM")
        {
            for(j=0;j<m_psdPtr->getSize();j++)
            {
                dotAl = wmTotj(j, T, P) / m_psdPtr->getmAvg(j); // [part/s/cm3] = [g/s/cm3] / [g/part]
                dotAlVector.push_back(dotAl);  // [part/s/cm^3]
            }
        }
        else if(m_massTransferType == "aggloTensor")
        {
            for(j=0;j<m_psdPtr->getSize();j++)
            {
                dotAl = tensorWmTotk(j, T, P) / m_psdPtr->getmAvg(j);  // [part/s/cm3] = [g/s/cm3] / [g/part]. here j is the destination bin
                dotAlVector.push_back(dotAl);  // [part/s/cm^3]
            }
        }
        else if(m_massTransferType == "kumar96")
        {
            for(j=0;j<m_psdPtr->getSize();j++)
            {
                dotAl = kumarAggloSource(j, T, P);
                dotAlVector.push_back(dotAl);  // [part/s/cm^3]
                
            }
            
        }
        
    }
    
    return dotAlVector;     // [part/s/cm^3]
}


void SootSourceTerm::setNuclType(string nuclType)
{
    m_nuclType = nuclType;
}

void SootSourceTerm::setAggloType(string aggloType)
{
    m_aggloType = aggloType;
}

void SootSourceTerm::setbetaConst(double betaConst)
{
    m_betaConst = betaConst;
}



void SootSourceTerm::setgammaNucl(double gammaNucl)
{
    m_gammaNucl = gammaNucl;
    m_nuclType = "gammaNucl";
}

void SootSourceTerm::setGrowthType(string growthType)
{
    m_growthType = growthType;
}

void SootSourceTerm::setg0(double g0)
{
    m_g0 = g0;
}

void SootSourceTerm::setTheta(double theta)
{
    m_theta = theta;
}

void SootSourceTerm::setb0(double b0)
{
    m_b0 = b0;
}

void SootSourceTerm::setx0n(double x0n)
{
    m_x0n = x0n;
}


// test function
void SootSourceTerm::showBoltzmann()
{
    double t = 1.0e-3;
    
    cout << endl;
    cout << endl;
    cout << "kb = " << boltzmann;
    cout << endl;
    cout << 1.0/3.0 << endl;
    cout << "T(t) = " << m_gasPhasePtr->getT(t);
    cout << endl;
    cout << endl;
}





double SootSourceTerm::betaFree(double m1, double m2, double T)
{
    double beta(0);
    
    double v1 = m1/rhoSoot;    // particle volume in [cm^3]. rhoSoot in [g/cm^3]. mass must be in [g]
    double v2 = m2/rhoSoot;
    
    beta = epsilon * pow(  3.0 / (4.0*pi), 1.0/6.0 ) * pow(  6.0*boltzmann*T / rhoSoot  , 1.0/2.0  ) * pow(  1.0/v1 + 1.0/v2  , 1.0/2.0  ) * pow(  pow(v1, 1.0/3.0) + pow(v2, 1.0/3.0)  , 2.0  );
    
    return beta;
}


double SootSourceTerm::betaFreeDc(double mi, double mj, double dci, double dcj, double T)
{
    double beta(0);
    double mij = mi * mj / (mi + mj);
    
    beta = epsilon * pow(pi*boltzmann*T/(2.0*mij),0.5) * pow((dci + dcj),2.0);
    return beta;
}

double SootSourceTerm::betaContinuumDc(double dc1, double dc2, double T, double P) // from Blanquart KIT (and Mauss KIT)
{
    double beta(0.0);
    double cunn1 = cunningham(T, P, dc1);
    double cunn2 = cunningham(T, P, dc2);
    double eta = dynVisco(T);
    
    beta = 2.0 * boltzmann * T / (3.0*eta) * (cunn1/dc1 + cunn2/dc2) * (dc1 + dc2);
    
    return beta;
}


double SootSourceTerm::betaHarmonicDc(double mi, double mj, double dci, double dcj, double T, double P)
{
    double betaFree = betaFreeDc(mi, mj, dci, dcj, T);
    double betaContinuum = betaContinuumDc(dci, dcj, T, P);
    double betaGeo = betaFree * betaContinuum / (betaFree+betaContinuum);
    
    return betaGeo;
}


double SootSourceTerm::betaNucl(double mDimer, double T)
{
    double beta(0);
    
    double vDimer = mDimer / rhoSoot;  // approx Dimer = sphere with density rhoSoot
    
    beta = epsilon * pow(  3.0 / (4.0*pi), 1.0/6.0 ) * pow(  6.0*boltzmann*T / rhoSoot  , 1.0/2.0  ) * pow(  1.0/vDimer + 1.0/vDimer  , 1.0/2.0  ) * pow(  pow(vDimer, 1.0/3.0) + pow(vDimer, 1.0/3.0)  , 2.0  );
    
    
    return beta;
}


double SootSourceTerm::betaCond(double mDimer, double m1, double T)
{
    double beta(0);
    double vDimer = mDimer / rhoSoot;
    double v1 = m1 / rhoSoot;   // in [cm3]
    
    beta = epsilonCond * pow(  3.0 / (4.0*pi), 1.0/6.0 ) * pow(  6.0*boltzmann*T / rhoSoot  , 1.0/2.0  ) * pow(  1.0/v1 + 1.0/vDimer  , 1.0/2.0  ) * pow(  pow(v1, 1.0/3.0) + pow(vDimer, 1.0/3.0)  , 2.0  );
    
    //cout << "beta = " << beta << "   vDimer = " << vDimer << "   v1 = " << v1 << endl;
    
    return beta;
}

double SootSourceTerm::betaCondDc(double mDimer, double m1, double dc1, double T)
{
    double beta(0);
    double mij = mDimer * m1 / (mDimer + m1);
    double oneThird = double(1.0/3.0);
    double vDimer = mDimer / rhoSoot;   // in [cm3]
    double dDimer = pow(6.0*vDimer/pi,oneThird);   // in [cm]
    
    beta = epsilonCond * pow(pi*boltzmann*T/(2.0*mij),0.5) * pow((dDimer + dc1),2.0);
    return beta;
}



// "reaction rate" collision BINk + BINj -> nu_j*BINj + nu_(j+1)*BINj+1. Only for agglo
double SootSourceTerm::w_kj(int k, int j, double T, double P)  // [collsions/s/cm3]
{
    double wkj(0);
    //double mAvgj = m_psdPtr->getmAvg(j);
    //double mAvgk = m_psdPtr->getmAvg(k);
    double nvj = m_psdPtr->getNv(j);
    double nvk = m_psdPtr->getNv(k);
    
    double betakj = betaAggloHomogeneousGasPhase(k, j, T, P);
    
    
    // !!! Nv not Nd is used for collision rates [part/volume]  !!!
    if(k==j)
    {
        wkj = 0.5 * betakj * nvj * nvk;  // "factor 1/2"
        // nvj and nvk in [part/cm^3]. wkj in [collisions/cm^3].
    }
    else
    {
        wkj = betakj * nvj * nvk;  // [collsions/s/cm3]
    }
    return wkj;
}

double SootSourceTerm::w_kj(int k, int j)  // only for solverType == "analytical"
{
    double wkj(0.0);
    if(m_solverType == "analytical")
    {
        double betakj = betaAggloAnalytical(k, j);
        
        double nvj = m_psdPtr->getNv(j);
        double nvk = m_psdPtr->getNv(k);
        
        if(k==j)
        {
            wkj = 0.5 * betakj * nvj * nvk;  // "factor 1/2" corresponds to 1 - delta_kj / 2
            // nvj and nvk in [part/cm^3]. wkj in [collisions/cm^3].
        }
        else
        {
            wkj = betakj * nvj * nvk;  // [collsions/s/cm3]
        }
    }
    else
    {
        cout << "ERROR: solverType"<< endl;
    }
    
    return wkj;
}


double SootSourceTerm::avgMbinkTojPlusOne(int k, int j)
{
    double mInfk = m_psdPtr->getmInf(k);
    double mSupk = m_psdPtr->getmSup(k);
    
    double avgM_k_jTojplusOne = 2.0/3.0*(pow(mSupk, 3.0)-pow(mInfk, 3.0))/(pow(mSupk, 2.0)-pow(mInfk, 2.0));
    
    return avgM_k_jTojplusOne;
}


double SootSourceTerm::avgMbinjTojPlusOne(int k, int j)
{
    double mSupj = m_psdPtr->getmSup(j);
    
    double avgM_j_jTojplusOne = mSupj -0.5 * avgMbinkTojPlusOne(k, j);
    
    return avgM_j_jTojplusOne;
}



double SootSourceTerm::avgMbinkToj(int k, int j)
{
    double Mkmin = m_psdPtr->getmInf(k);  // Mkmin is mInfk
    double Mkmax = m_psdPtr->getmSup(k);   // Mkmax is mSupk
    double Mjmin = m_psdPtr->getmInf(j);
    double Mjmax = m_psdPtr->getmSup(j);
    double avgM_k_jToj = ((Mjmax-Mjmin)*(pow(Mkmax, 2.0)/2.0-pow(Mkmin, 2.0)/2.0)-(pow(Mkmax, 3.0)/3.0-pow(Mkmin, 3.0)/3.0))/((Mjmax-Mjmin)*(Mkmax-Mkmin)-(pow(Mkmax, 2.0)/2.0-pow(Mkmin, 2.0)/2.0));
    
    return avgM_k_jToj;
}



double SootSourceTerm::aCoef(int k, int j)
{
    double mAvgk = m_psdPtr->getmAvg(k);
    double mInfj = m_psdPtr->getmInf(j);
    double mSupj = m_psdPtr->getmSup(j);
    
    double aCoefficient(1);
    if(k<j)
    {
        aCoefficient = 1 - mAvgk/(mSupj-mInfj);
    }
    else
    {
        cout << "error aCoef";
    }
    
    
    return aCoefficient;
}




double SootSourceTerm::nuj(int k, int j)
{
    
    double mAvgj = m_psdPtr->getmAvg(j);
    
    double nu_j = (aCoef(k, j)*avgMbinkToj(k, j) - (1-aCoef(k, j))*avgMbinjTojPlusOne(k, j))/(mAvgj) + 1;
    
    return nu_j;
}



double SootSourceTerm::nujPlusOne(int k, int j)
{
    
    double nu_jPlusOne(0);
    int jPlusOne = j+1;
    double mAvgjPlusOne = m_psdPtr->getmAvg(j+1);
    
    if(jPlusOne<m_psdPtr->getSize())
    {
        nu_jPlusOne = (1-aCoef(k, j)) * ( avgMbinkTojPlusOne(k, j) + avgMbinjTojPlusOne(k, j) ) / (mAvgjPlusOne);
    }
    else
    {
        nu_jPlusOne = 0;
        cout << "error nujPlusOne j+1 > lVectorsize " << endl;
    }
    
    return nu_jPlusOne;
}



double SootSourceTerm::wmNegRj(int k, int j, double T, double P)
{
    
    double wmNeg_Rj(0);
    double mAvgj = m_psdPtr->getmAvg(j);
    
    if(k<j)
    {
        wmNeg_Rj = -mAvgj* w_kj(k, j, T, P);   // [g/s]
        
    }
    if(k==j)
    {
        wmNeg_Rj = -mAvgj* w_kj(k, j, T, P);  // for k=j -1 but summing with wmNegRk will give -2. but factor 1/2 for w_kj. compensate
    }
    if(k>j)
    {
        cout << "error k>j " << endl;
        wmNeg_Rj = 0;
    }
    
    return wmNeg_Rj;
}



double SootSourceTerm::wmNegRk(int k, int j, double T, double P)
{
    double wmNeg_Rk(0);
    double mAvgk = m_psdPtr->getmAvg(k);
    
    if(k<j)
    {
        wmNeg_Rk = -mAvgk*w_kj(k, j, T, P);    // [g/s]
        
    }
    if(k==j)
    {
        wmNeg_Rk = -mAvgk*w_kj(k, j, T, P);  // for k=j -2 but factor 1/2 for w_kj. compensate
    }
    if(k>j)
    {
        cout << "error k>j " << endl;
        wmNeg_Rk = 0;
    }
    
    return wmNeg_Rk;
}


double SootSourceTerm::wmPosRj(int k, int j, double T, double P)
{
    double mAvgj = m_psdPtr->getmAvg(j);
    double wmPosR_j(0);
    if(k<j)
    {
        wmPosR_j = nuj(k, j)* mAvgj * w_kj(k, j, T, P);
        
    }
    if(k==j)
    {
        wmPosR_j = 0;
    }
    if(k>j)
    {
        cout << "error k>j " << endl;
        wmPosR_j = 0;
    }
    
    return wmPosR_j;
}


double SootSourceTerm::wmPosPlusOneRj(int k, int jPlusOne, double T, double P)
{
    int j = jPlusOne -1;
    
    double wmPosR_jPlusOne(0);
    double mAvgjPlusOne = m_psdPtr->getmAvg(jPlusOne);
    
    if(jPlusOne>0)
    {
        if(k<j)
        {
            
            wmPosR_jPlusOne += nujPlusOne(k, j) * mAvgjPlusOne * w_kj(k, j, T, P);   // case k < j
            
            
        }
        if(k==j)
        {
            wmPosR_jPlusOne += 1 * mAvgjPlusOne * w_kj(k, j, T, P);  // case k = j
        }
        if(k>j)
        {
            cout << "error k>j " << endl;
            wmPosR_jPlusOne = 0;
        }
    }
    else
    {
        cout << "error jPlusOne = 0" << endl;
        wmPosR_jPlusOne = 0;
    }
    
    return wmPosR_jPlusOne;
}



double SootSourceTerm::wmNegJ(int j, double T, double P)
{
    double wmNegJ(0);
    int k=0;
    for(k=0; k<j; k++)      // case collision of j with a smaller particle
    {
        wmNegJ += wmNegRj(k, j, T, P);
    }
    
    wmNegJ += wmNegRj(k, j, T, P) + wmNegRk(k, j, T, P); //k=j
    
    for(k=(j+1);(k+1)<m_psdPtr->getSize(); k++) // case collision of j with a bigger particle. only possible if smaller than biggest size  -> for until (k+1)<psd.size
    {
        wmNegJ += wmNegRk(j, k, T, P);
    }
    
    return wmNegJ;
}



double SootSourceTerm::wmPosJ(int j, double T, double P)
{
    double wmPosj(0);
    int k(0);
    for(k=0; k<j; k++)
    {
        wmPosj += wmPosRj(k, j, T, P);
    }
    
    //k=j
    wmPosj += wmPosRj(k, j, T, P); // = 0 in this case
    
    return wmPosj;
}


double SootSourceTerm::wmPosJPlusOne(int jPlusOne, double T, double P)
{
    double wmPosjPlus(0);
    int k(0);
    int j = jPlusOne - 1;
    
    if(jPlusOne>0)
    {
        for(k=0; k<j;k++)
        {
            wmPosjPlus += wmPosPlusOneRj(k, (j+1), T, P);
        }
        wmPosjPlus += wmPosPlusOneRj(k, (j+1), T, P);  // k=j
    }
    else
    {
        wmPosjPlus = 0;
    }
    
    return wmPosjPlus;
}


double SootSourceTerm::wmTotj(int j, double T, double P)
{
    double wmTotj(0);
    
    int jPlusOne = j+1;
    if(jPlusOne<m_psdPtr->getSize())
    {
        wmTotj = wmNegJ(j, T, P) + wmPosJ(j, T, P) + wmPosJPlusOne(j, T, P);
    }
    else
    {
        wmTotj = wmPosJPlusOne(j, T, P);
        
    }
    
    return wmTotj;   // [g/s]
}


vector<double> SootSourceTerm::deltamTotGrowthVector(double t)
{
    vector<double> deltamTotVec;
    double deltam(0.0);
    
    if(m_solverType == "analytical")
    {
        int nbBins = m_psdPtr->getSize();
        int i(0);
        for(i=0;i<nbBins;i++)
        {
            //double mi = m_psdPtr->getmAvg(i);
            double mi = m_psdPtr->getmAvgVar(i);
            deltam = m_g0 * pow(mi,m_theta);
            deltamTotVec.push_back(deltam);
        }
    }
    else if(m_solverType == "HomogeneousGasPhase")
    {
        if(m_growthType=="standard")
        {
            
            int nbBins = m_psdPtr->getSize();
            int i(0);
            for(i=0;i<nbBins;i++)
            {
                deltam = deltamCondi(t, i) + deltamSurfi(t, i);
                deltamTotVec.push_back(deltam);
                //cout << "in SootSourceTerm, for i = " << i << "  deltam  = " << deltam << "   deltamCond = " << deltamCondi(t, i) << "   deltamSurf = " << deltamSurfi(t, i) << endl << endl;
            }
        }
        
        
        else if(m_growthType=="G0mTheta")
        {
            int nbBins = m_psdPtr->getSize();
            int i(0);
            for(i=0;i<nbBins;i++)
            {
                //double mi = m_psdPtr->getmAvg(i);
                double mi = m_psdPtr->getmAvgVar(i);
                deltam = m_g0 * pow(mi,m_theta);
                deltamTotVec.push_back(deltam);
            }
        }
    }
    
    return deltamTotVec;
}

double SootSourceTerm::deltamTotGrowthPart(double t, double m)
{
    double deltam(0.0);
    
    if(m_solverType == "analytical")
    {
        deltam = m_g0 * pow(m,m_theta);
    }
    else if(m_solverType == "HomogeneousGasPhase")
    {
        if(m_growthType=="standard")
        {
            int bin = findBin(m);
            deltam = deltamCondi(t, bin) + deltamSurfi(t, bin);
        }
        else if(m_growthType=="G0mTheta")
        {
            deltam = m_g0 * pow(m, m_theta);
        }
    }
    return deltam;
}


vector<double> SootSourceTerm::testCaseNuclSourceVector()
{
    vector<double> nuclSourceVector;
    
    if(m_nuclType == "exponential")
    {
        int nBin = m_psdPtr->getSize();
        int i(0);
        for(i=0;i<nBin;i++)
        {
            double mass = m_psdPtr->getmAvg(i);
            double mInf = m_psdPtr->getmInf(i);
            double mSup = m_psdPtr->getmSup(i);
            double deltaL = mSup - mInf;
            
            double dotB = m_b0 / m_x0n * exp(- mass/ m_x0n) * deltaL;
            nuclSourceVector.push_back(dotB);
        }
    }
    
    else
    {
        cout << "error m_nuclType exponential" << endl;
    }
    
    return nuclSourceVector;
}


double SootSourceTerm::dimerConcentration(double t)   // [part/cm3]
{
    // from KIT Blanquart, Pitsch, p476  eq (16)
    // Solves QSS. Solution of a quadratic equation.
    // partDimerSource = betaNucl * NDimer^2 + sigma(betaCondi * Nvi) * NDimer
    // equivalent to a*x^2 + b*x + c = 0 with a = betaNucl; b = sigma(betaCondi * Nvi); c = - partDimerSource
    
    double c = - partDimerSource(t);
    
    double T = m_gasPhasePtr->getT(t);
    
    double mDimer = 32.0 * cmass;       // if dimerization is from 2 pyrene molecules
    double a = betaNucl(mDimer, T);
    
    int psdSize = m_psdPtr->getSize();
    
    double b(0.0);
    int i(0);
    for(i=0;i<psdSize;i++)
    {
        double mi = m_psdPtr->getmAvg(i);
        double Nvi = m_psdPtr->getNv(i);
        
        double bi(0.0);
        
        if(m_aggloType=="collisionDiameter")
        {
            double dci = collisionDiameterCalc(mi);  // in [cm]
            
            bi = betaCondDc(mDimer, mi, dci, T) * Nvi;
        }
        else if(m_aggloType == "harmonicAvgDc")
        {
            double dci = collisionDiameterCalc(mi);  // //  dc in [cm]
            double P = m_gasPhasePtr->getPperfectGas(t);
            
            double vDimer = mDimer / rhoSoot;  // in [cm3]
            double oneThird = 1.0/3.0;
            double d3 = vDimer*6.0/pi;
            double dcDimer = pow(d3, oneThird); // in [cm] from sphere for dimers/pyrene
            
            bi = betaHarmonicDc(mi, mDimer, dci, dcDimer, T, P);
        }
        else
        {
            bi = betaCond(mDimer, mi, T) * Nvi;
        }
        
        b+= bi;
    }
    
    // now we have values for a, b, c. We can calculate delta and the positive root
    double delta = pow(b, 2.0) - 4.0 * a * c;
    double x1(0.0);
    double x2(0.0);
    
    if(delta >0.0)
    {
        x1 = (-b + pow(delta, 0.5))/ (2.0 * a);
        x2 = (-b - pow(delta, 0.5))/ (2.0 * a);
        
        if(x1>=0.0 && x2 <=0.0)
        {
        }
        else
        {
            cout << "ERROR [Dimer] calculation: roots sign" << endl;
            x1 = 0.0;
        }
    }
    else if(delta == 0.0)
    {
        x1 = -b / (2.0*a);
        if(x1>=0.0)
        {
        }
        else
        {
            cout << "ERROR [Dimer] calculation: roots sign" << endl;
            x1 = 0.0;
        }
    }
    else
    {
        cout << "ERROR [Dimer] calculation: delta sign" << endl;
        x1 = 0.0;
    }
    
    return x1;
}


double SootSourceTerm::partCondSourcei(double t, int i)  // dotHc
{
    double dotCondi(0.0);
    
    double T = m_gasPhasePtr->getT(t);
    //double mAvgi = m_psdPtr->getmAvg(i);
    double mAvgVari = m_psdPtr->getmAvgVar(i);
    double Nvi = m_psdPtr->getNv(i);  // [part/cm3]
    
    double mPAH = 16.0*cmass;
    double YPAH(0.0);
    
    if(m_PAHType == "nofeedback")
    {
        YPAH = m_gasPhasePtr->getYA4(t);  // no consumption of pyrene in gas phase
    }
    else if(m_PAHType == "corrected")
    {
        YPAH = YA4corrected(t);     // approximation of YA4 - YA4 consumed
        //cout << "i = " << i << "   YPAH = " << YPAH << endl;
    }
    else if(m_PAHType == "hybrid")
    {
        YPAH = YA4hybrid(t);        // YA4 imposed for the range of data available (from paper's graph and corrected for smaller x)
    }
    else
    {
        cout << "ERROR m_PAHType" << endl;
        YPAH = 0.0;
    }
    
    
    double rho = m_gasPhasePtr->getrho(t);
    double rhoGramm = rho * 1e3 *1e-6; // [in g/cm3]
    double molarMass = 16.0 * cmass * avogadro;
    double cPAH = YPAH * rhoGramm / molarMass;  // [mol/cm3]
    double NPAH = cPAH *avogadro;    // [part/cm3]
    
    if(m_nuclType == "kraft")
    {
        double beta(0.0);
        if(m_aggloType == "collisionDiameter")
        {
            //double dci = collisionDiameterCalc(mAvgi);  // in [cm]
            //beta = betaCondDc(mPAH, mAvgi, dci, T);
            double dci = collisionDiameterCalc(mAvgVari);  // in [cm]
            beta = betaCondDc(mPAH, mAvgVari, dci, T);
        }
        else if(m_aggloType == "harmonicAvgDc")
        {
            //double dci = collisionDiameterCalc(mAvgi);  // in [cm]
            double dci = collisionDiameterCalc(mAvgVari);  // in [cm]
            
            double vPAH = mPAH / rhoSoot;  // in [cm3]
            double oneThird = 1.0/3.0;
            double d3 = vPAH*6.0/pi;
            double dcPAH = pow(d3, oneThird); // in [cm] from sphere for dimers/pyrene
            
            double P = m_gasPhasePtr->getPperfectGas(t);
            
            //beta = betaHarmonicDc(mPAH, mAvgi, dcPAH, dci, T, P);
            beta = betaHarmonicDc(mPAH, mAvgVari, dcPAH, dci, T, P);
        }
        else
        { // if aggloType == "standard" or other
            //beta = betaCond(mPAH, mAvgi, T);
            beta = betaCond(mPAH, mAvgVari, T);
        }
        
        dotCondi = beta * Nvi * NPAH;    // [molecPAH/cm3/s]  from D'Aubagnac
        //cout << "i = " << i << "dotCondi = " << dotCondi  << "   beta = " << beta << "  Nvi = " << Nvi << "   NPAH = " << NPAH << endl;
    }
    else
    {
        double mDimer = 32.0 * cmass;   // dimerisation of pyrene
        
        double beta(0.0);
        
        if(m_aggloType == "collisionDiameter")
        {
            
            //double dAvgi = collisionDiameterCalc(mAvgi);
            //beta = betaCondDc(mDimer, mAvgi, dAvgi, T);
            
            double dAvgi = collisionDiameterCalc(mAvgVari);
            beta = betaCondDc(mDimer, mAvgVari, dAvgi, T);
        }
        else if(m_aggloType == "harmonicAvgDc")
        {
            //double dci = collisionDiameterCalc(mAvgi);  // in [cm]
            double dci = collisionDiameterCalc(mAvgVari);  // in [cm]
            
            double vDimer = mDimer / rhoSoot;  // in [cm3]
            double oneThird = 1.0/3.0;
            double d3 = vDimer*6.0/pi;
            double dcDimer = pow(d3, oneThird); // in [cm] from sphere for dimers/pyrene
            
            double P = m_gasPhasePtr->getPperfectGas(t);
            
            //beta = betaHarmonicDc(mDimer, mAvgi, dcDimer, dci, T, P);
            beta = betaHarmonicDc(mDimer, mAvgVari, dcDimer, dci, T, P);
        }
        else
        {
            //beta = betaCond(mDimer, mAvgi, T);
            beta = betaCond(mDimer, mAvgVari, T);
        }
        
        double dimerC = dimerConcentration(t);    // [part/cm3]
        
        dotCondi = beta * Nvi * dimerC;  // [dimers/cm3/s]
    }
    
    //cout << " i = " << i << "   dotCondi = " << dotCondi << endl;
    return dotCondi;  // [dimers/cm3/s]
}


double SootSourceTerm::deltamCondi(double t, int i)  // [g/part/s]. G_cond(l^*,Y)
{
    double dotCondi = partCondSourcei(t, i);  // [dimers/cm3/s] or // [Pyrene/cm3/s]. dotH_c
    double dotm(0.0);
    if(m_nuclType =="kraft")
    {
        double mPAH = 16.0 *cmass;  // direct condensation of pyrene   [g/dimer]
        double dotmCondBini = mPAH * dotCondi;
        double Nvi = m_psdPtr->getNv(i);   // [part/cm3]
        
        if (Nvi == 0.0) {
            dotm = 0.0;
        } else{
            dotm = dotmCondBini / Nvi;  // [g/part/s]
        }
        
        
    }
    else
    {
        double mDimer = 32.0 * cmass;   // dimerisation of pyrene   [g/dimer]
        double dotmCondBini = mDimer * dotCondi;   // [g/cm3/s]
        double Nvi = m_psdPtr->getNv(i);   // [part/cm3]
        
        if (Nvi == 0.0) {
            dotm = 0.0;
        } else{
            dotm = dotmCondBini / Nvi;  // [g/part/s]
        }
    }
    
    return dotm;   // !! still needs to be multiplicated by timePerIt in [s] (done in NpSolver)!!
}


int SootSourceTerm::findBin(double m)
{
    int i(0);
    int bin(0);
    int nBins = m_psdPtr->getSize();
    int iMax = nBins-1;
    double mSupMax = m_psdPtr->getmSup(iMax);
    if(m<=mSupMax)
    {
        double mSup = m_psdPtr->getmSup(i);
        while(m>mSup)
        {
            i++;
            mSup = m_psdPtr->getmSup(i);
        }
        bin = i;
    }
    else
    {
        bin = 0;
        cout << "ERROR findBin, mass looked for > mMax" << endl;
    }
    return bin;
}


double SootSourceTerm::concentrationActiveSites(double t, int bini)
{
    double surfAvgi(0.0);
    if(m_surfType == "sphere")
    {
        surfAvgi = m_psdPtr->getsAvg(bini);   // cm2. from sphere
    }
    else if(m_surfType == "fractal")
    {
        surfAvgi = surfFromFractal(bini);
    }
    else
    {
        cout << "ERROR m_surfType" << endl;
    }
    
    double alphaHacaBini = alphaHacaBin(bini, t);
    double nbSitesPerPart = chi * surfAvgi;      // [sites/part]
    double nbActiveSitesPerPart = alphaHacaBini * nbSitesPerPart;   // [active sites / part]
    double Nvi = m_psdPtr->getNv(bini);    // [part/cm3]
    double cAiPlusRi = Nvi * nbActiveSitesPerPart;    // [active sites / cm3]
    double molarcAiPlusRi = cAiPlusRi / avogadro;    // [mol of active sites / cm3]
    return molarcAiPlusRi;    // [mol of active sites / cm3]
}

double SootSourceTerm::alphaHacaBin(int bini, double t)
{
    double alphaHacaBini(0.0);
    
    if(m_hacaType == "standard")
    {
        m_alphaHACA = 1.0;
        alphaHacaBini = m_alphaHACA;  // same for all bins
    }
    else if (m_hacaType == "constant")
    {
        alphaHacaBini = m_alphaHACA;  // same for all bins
    }
    else if (m_hacaType == "appel")
    {
        // from Appel and from berkeley code. In Berkeley code, made with MOM so need for an average mu1 but here we can do it for bin size

        
        //double mAvg = m_psdPtr->getmAvg(bini);
        double mAvgVar = m_psdPtr->getmAvgVar(bini);
        
        //double mNormalized = mAvg / cmass;  // nb of Carbon per particle in bini [carbon/part]
        double mNormalized = mAvgVar / cmass;  // nb of Carbon per particle in bini [carbon/part]
        
        
        double T = m_gasPhasePtr->getT(t);
        double a = 12.65 - 0.00563*T;
        double b = -1.38 + 0.00068 *T;
        //in Appel only one avg alpha for all sizes, function of avg size. alphaHACA = tanh(a/log10(mu1) +b);
        alphaHacaBini = tanh(a/log10(mNormalized) +b);  // different for each bin
        
    }
    else if(m_hacaType == "appelAndFactor")
    {
        //double mAvg = m_psdPtr->getmAvg(bini);
        double mAvgVar = m_psdPtr->getmAvgVar(bini);
        //double mNormalized = mAvg / cmass;  // nb of Carbon per particle in bini [carbon/part]
        double mNormalized = mAvgVar / cmass;  // nb of Carbon per particle in bini [carbon/part]
        
        
        double T = m_gasPhasePtr->getT(t);
        double a = 12.65 - 0.00563*T;
        double b = -1.38 + 0.00068 *T;
        //in Appel only one avg alpha for all sizes, function of avg size. alphaHACA = tanh(a/log10(mu1) +b);
        alphaHacaBini = 1.2 * tanh(a/log10(mNormalized) +b);  // different for each bin
    }
    else
    {
        cout << "error hacaType" << endl;
        m_alphaHACA = 0.0;
        alphaHacaBini = m_alphaHACA;   // same for all bins
    }
    return alphaHacaBini;
}

double SootSourceTerm::avgalphaHaca(double t)
{
    double avgAlpha(0.0);
    
    if(m_hacaType == "standard")
    {
        m_alphaHACA = 1.0;
        avgAlpha = m_alphaHACA;  // same for all bins
    }
    else if (m_hacaType == "constant")
    {
        avgAlpha = m_alphaHACA;  // same for all bins
    }
    else if (m_hacaType == "appel" || m_hacaType == "appelAndFactor")
    {
        double sumNvAlpha(0.0);
        int i(0);
        int nbBins = m_psdPtr->getSize();
        for(i=0;i<nbBins;i++)
        {
            double alphaBini = alphaHacaBin(i, t);
            double Nvi = m_psdPtr->getNv(i);
            double NvAlphai = alphaBini*Nvi;
            sumNvAlpha+=NvAlphai;
        }
        avgAlpha = sumNvAlpha / m_psdPtr->getNvT();
    }
    else
    {
        cout << "error hacaType" << endl;
        m_alphaHACA = 0.0;
        avgAlpha = m_alphaHACA;   // same for all bins
    }

    
    return avgAlpha;
}


double SootSourceTerm::kCoefReactionRates(double A, double n, double E, double t)
{
    double T = m_gasPhasePtr->getT(t);
    double k = A * pow(T, n) * exp(-E/(Rconst * T));
    return k;
}

double SootSourceTerm::concentrationRadicalSites(double t, int bini)
{
    // calculation of reaction constants k in [cm3 mol-1 s-1]
    double kf1 = kCoefReactionRates(fR1_A, fR1_n, fR1_E, t);
    double kr1 = kCoefReactionRates(rR1_A, rR1_n, rR1_E, t);
    double kf2 = kCoefReactionRates(fR2_A, fR2_n, fR2_E, t);
    double kr2 = kCoefReactionRates(rR2_A, rR2_n, rR2_E, t);
    double kf3 = kCoefReactionRates(fR3_A, fR3_n, fR3_E, t);
    double kf4 = kCoefReactionRates(fR4_A, fR4_n, fR4_E, t);
    double kf5 = kCoefReactionRates(fR5_A, fR5_n, fR5_E, t);
    //double kf6 = kCoefReactionRates(fR6_A, fR6_n, fR6_E, t);
    
    // calculation of species concentrations in [mol/cm3] from species mass fracions
    double rho = m_gasPhasePtr->getrho(t);
    double cH = m_gasPhasePtr->getMolarConcentration(t, rho, "H");
    double cH2 = m_gasPhasePtr->getMolarConcentration(t, rho, "H2");
    double cOH = m_gasPhasePtr->getMolarConcentration(t, rho, "OH");
    double cH2O = m_gasPhasePtr->getMolarConcentration(t, rho, "H2O");
    double cC2H2 = m_gasPhasePtr->getMolarConcentration(t, rho, "C2H2");
    double cO2 = m_gasPhasePtr->getMolarConcentration(t, rho, "O2");
    
    // calculation of surface reaction rates in [mol/cm3/s] (strictly speaking not yet because multiplied by [Ri] and [Ai] after)
    double fR1 = kf1 * cH;
    double rR1 = kr1 * cH2;
    
    double fR2 = kf2 * cOH;
    double rR2 = kr2 * cH2O;
    
    double fR3 = kf3 * cH;
    double fR4 = kf4 * cC2H2;
    double fR5 = kf5 * cO2;
    //double fR6 = kf6 * cOH;
    
    // then QSS assumption in order to get [Ri]
    double denom = rR1 + rR2 + fR3 + fR4 + fR5;  // reactions consuming [Ri]
    double qssRatio(1.0);
    if(denom != 0.0)
    {
        double numerator = fR1 + fR2;   // reactions producing [Ri]
        qssRatio = numerator / denom;
    }
    else
    {
        cout << "ERROR: denominator for [Ri] QSS = 0" << endl;
    }
    
    double cAiPlusRi = concentrationActiveSites(t, bini);  // [mol/cm3]
    double cRi = qssRatio * cAiPlusRi;
    return cRi;   // [mol/cm3]
}


double SootSourceTerm::surfMassSource(double t, int bini)    //dotw_haca,Ox(l^*)
{
    // calculation of reaction constants k in [cm3 mol-1 s-1]
    double kf4 = kCoefReactionRates(fR4_A, fR4_n, fR4_E, t);
    double kf5 = kCoefReactionRates(fR5_A, fR5_n, fR5_E, t);
    double kf6 = kCoefReactionRates(fR6_A, fR6_n, fR6_E, t);
    
    // calculation of species concentrations in [mol/cm3] from species mass fracions
    double rho = m_gasPhasePtr->getrho(t);
    double cOH = m_gasPhasePtr->getMolarConcentration(t, rho, "OH");
    double cC2H2 = m_gasPhasePtr->getMolarConcentration(t, rho, "C2H2");
    double cO2 = m_gasPhasePtr->getMolarConcentration(t, rho, "O2");
    
    // calculation of surface reaction rates in [mol/cm3/s] (strictly speaking not yet because multiplied by [Ri] and [Ai] after)
    double fR4 = kf4 * cC2H2;
    double fR5 = kf5 * cO2;
    double fR6 = kf6 * cOH;
    
    double cRi = concentrationRadicalSites(t, bini);       // [mol/cm3]. [R_l^*]
    double cAiPlusRi = concentrationActiveSites(t, bini);  // [mol/cm3]. [S_l^*]

    double rateC2H2add = fR4 * cRi;  // [mol/cm3/s]
    double rateO2oxi = fR5 * cRi;
    double rateOHoxi = fR6 * cAiPlusRi;   // in D'Aubagnac, cAi only. could use (cAiPlusRi - cRi). Anyway difference should be small as cRi << cAi
    
    // cmass is in [g/atom]
    double cMolarMass = avogadro * cmass;  // in [g/mol]
    
    double massRateC2H2add = 2.0 * cMolarMass * rateC2H2add;  // in [g/cm3/s]
    double massRateO2oxi = -2.0 * cMolarMass * rateO2oxi;
    double massRateOHoxi = -1.0 * cMolarMass * rateOHoxi;
    
    double massRateSurf = massRateC2H2add + massRateO2oxi + massRateOHoxi;
    
    return massRateSurf;   // in [g/cm3/s]
}


double SootSourceTerm::deltamSurfi(double t, int i)   // G_surf(l^*,t)
{
    double massRateSurf = surfMassSource(t, i);  // in [g/cm3/s].  dotw_haca,Ox(l^*)
    double Nvi = m_psdPtr->getNv(i);   // in [part/cm3]
    
    double deltam(0.0);
    if (Nvi == 0.0) {
        deltam = 0.0;
    } else {
        deltam = massRateSurf / Nvi;  // in [g/part/s]
    }
    return deltam;
}


void SootSourceTerm::addConsoTotPAH(double timePerIt, double t)
{
    // dotmPAHcond and m_dotmPAHnucl in [g/cm3/s]
    double mPAH = 16.0 * cmass;
    double dotmPAHcond = partCondSourceTot(t) * mPAH;  // [g/cm3/s] = [molecPAH/cm3/s] * [g/molec]
    
    double deltam = (m_dotmPAHNucl + dotmPAHcond) * timePerIt;  // in [g/cm3]
    m_consoTotPAH += deltam; // in [g/cm3]
}


double SootSourceTerm::YA4corrected(double t)
{
    double YA4corr(0.0);
    
    double rho = m_gasPhasePtr->getrho(t);  // [kg/m3]
    double rhoGramm = rho * 1.0e3 * 1.0e-6; // [g/cm3]
    
    if(m_A4correctionType == "consoTot")
    {
        double YA4init = m_gasPhasePtr->getYA4(t);
        
        
        YA4corr = YA4init - m_consoTotPAH / rhoGramm;   // consoTotPah in [g/cm3]
         // problem: as YA4init and consoTot are about 2 orders of magnitude bigger than YA4corrected, and are "stiff", this is not so accurate to do this way. "discontinuities" of YA4corrected are observed for high x as YA4init is high and stiff(important source termof YA4initProduction, which is calculated only in cantera) at these values of x.
        // solution: estimate YA4init source term wYA4. We can estimate it from cantera output for each value of x/t. Then we could do something like rho(it+1) * YA4corr(it+1) = rho(it) * [ YA4corr(it) + wYA4(it) - wYA4cond - wYA4Nucl ]        (wYA4cond and wYA4Nucl calculated from dotPAHNucl and dotPAHCond
        // if needed, refine time steps if source terms are too big. criteria like wYA4 / YA4corrected < constant
    }
    else if(m_A4correctionType == "PAHsources")
    {
        YA4corr = m_CmA4corrected / rhoGramm;
        //cout << "CmA4corrected = " << m_CmA4corrected  << "   rho = " << rhoGramm << endl;
    }
    else if(m_PAHType == "corrected")
    {
        cout << "ERROR PAHType or A4correctionType" << endl;
    }
    
    return YA4corr;
}

double SootSourceTerm::YA4hybrid(double t)
{
    double YPAH(0.0);
    double tinitImposed = 4.9e-3;
    if(t<tinitImposed)
    {
        YPAH = YA4corrected(t);
    }
    else
    {
        YPAH = m_gasPhasePtr->getYA4imposed(t);
    }
    return YPAH;
}

double SootSourceTerm::getConsoTotPAH()
{
    return m_consoTotPAH;
}

HomogeneousGasPhase* SootSourceTerm::getGasPhasePtr()
{
    return m_gasPhasePtr;
}

void SootSourceTerm::setalphaHaca(double alpha)
{
    m_alphaHACA = alpha;
}

double SootSourceTerm::getAlphaHacaUniform()
{
    return m_alphaHACA;
}

void SootSourceTerm::setHacaType(string hacaType)
{
    m_hacaType = hacaType;
}

void SootSourceTerm::setPAHType(string PAHType)
{
    m_PAHType = PAHType;
}


double SootSourceTerm::getDotmPAHnucl()
{
    return m_dotmPAHNucl;
}

double SootSourceTerm::partCondSourceTot(double t)
{
    double dotCondTot(0.0);
    int nBins = m_psdPtr->getSize();
    int i(0);
    for(i=0;i<nBins;i++)
    {
        dotCondTot += partCondSourcei(t, i);  // [molecPAH/cm3/s]
    }
    return dotCondTot;  // [molecPAH/cm3/s]
}

void SootSourceTerm::setA4correctionType(string correctionType)
{
    m_A4correctionType = correctionType;
}


double SootSourceTerm::getDotCmA4correctedSource(double t) // in [g/cm3/s]
{
    double mPAH = 16.0*cmass;
    
    double wmA4tot(0.0);
    double wmPlusA4 = m_gasPhasePtr->getwmA4(t);  // in [g/cm3/s]
    double wmNuclA4 = - m_dotmPAHNucl;   // in [g/cm3/s]
    double wmCondA4 = - partCondSourceTot(t) * mPAH;
    wmA4tot = wmPlusA4 + wmNuclA4 + wmCondA4;   // in [g/cm3/s]
    
    return wmA4tot;
    
}

void SootSourceTerm::advanceCmA4correctedSource(double wmA4perIt)
{
    m_CmA4corrected += wmA4perIt;  // in [g/cm3]
}

double SootSourceTerm::getCmA4()
{
    return m_CmA4corrected;
}

void SootSourceTerm::setKf(double kf)
{
    m_kf = kf;
}

void SootSourceTerm::setImposedDpSource(double dp)
{
    m_imposedDpSource = dp;
}

void SootSourceTerm::setDf(double Df)
{
    m_Df = Df;
}

double SootSourceTerm::kfInterpolation(int bini)
{
    double kf(1.0);
    
    /*
    double interpolationFactor = 10.0;
    
    double mAvgi = m_psdPtr->getmAvg(bini);
    double vPrimary = pi / 6.0 * pow(m_imposedDpSource,3.0); // in [nm3]
    double vPrimarycm3 = vPrimary * 1.0e-21;   // in [cm3].
    double mPrimary = rhoSoot * vPrimarycm3;   // rhoSoot in [g/cm3]. mPrimary in [g]
    double interpolationSup = interpolationFactor * mPrimary;
    
    if(mAvgi<=mPrimary)
    {
        kf=1.0;
    }
    else if(mAvgi>interpolationSup)
    {
        kf = m_kf;
    }
    else
    {
        
        // interpolation on kf is from sphere k^(1/D) = (np+8)/9  for 1<= np <= 10 and k^(1/D) = 2 for np>10
        // "smooth transtion" from Patterson Thesis p100
        double np = mAvgi / mPrimary;
        double coef = (np+8.0) / 9.0 / 2.0 * pow(m_kf,1.0/m_Df); // initially made to make kf^(1/Df) tend to 2. Here kf^(1/Df) tends to m_kf^(1/Df)
        kf = pow(coef, m_Df);
        
    }
    */
    
    // kf discontinuous but interpolation on dc.
    kf=m_kf;
    
    return kf;
}

double SootSourceTerm::collisionDiameterCalc(double mass)  // mass in [g], dc in [cm]
{
    // Dc ~ 1.29 * Dg. Assumption that Dc is proportional to Dg. coefficient taken as the one verified for spheres. (cf Patterson, Kazakov/Frenklach, Kruis). Kazakov/Frenklach Followed Kruis et al for 2 approx: dm = dc (used for betaContinuum) + Cunningham slip correction factors evaluated as a function of Kn itself evaluated based on Dc + chose the coefficient of proportionality between dc and Rg in such a way that in the limit of a single spherical particle, dc becomes equal to the particle diameter, i.e. dc = dg * pow(5.0/3/0, 0.5). Dc ~ 1.29 * Dg
    
    double transitionFactor = 10.0;
    double dci(1.0);
    
    int i = findBin(mass);
    
    double dpNano = m_imposedDpSource;  // in [nm]
    double dp = dpNano * 1.0e-7;  // in [cm]
    double df = m_Df;
    
    double vPrimary = pi / 6.0 * pow(dpNano,3.0); // in [nm3]
    double vPrimarycm3 = vPrimary * 1.0e-21;   // in [cm3].
    double mPrimary = rhoSoot * vPrimarycm3;   // rhoSoot in [g/cm3]. mPrimary in [g]
    
    double npi = mass / mPrimary;
    
    if(npi<=1) // sphere
    {
        double dcNano = m_psdPtr->getdAvgNano(i);  // in [nm]
        dci = dcNano * 1.0e-7; // in [cm]
    }
    else if(npi<=transitionFactor)  // transition from dc = dSphere to dc = dCollision
    {
        double kfi = kfInterpolation(i);
        double dg = dp * pow(npi/kfi, 1.0/df);  // in [cm]. Dg from fractal relation
        double dcoli = dg * pow(5.0/3.0, 0.5);   // in [cm]. Dc ~ 1.29 * Dg
        
        double dSphereNano = m_psdPtr->getdAvgNano(i);  // in [nm]
        double dSphere = dSphereNano * 1.0e-7; // in [cm]
        
        if(npi<=(transitionFactor/2.0))
        {
            double xTransition = -2.0 + (npi-1.0)/(transitionFactor/2.0-1.0) *2.0;   // must vary from -2 to 0 for np from 1 to transitionFactor/2
            
            dci = (dcoli + dSphere) / 2.0 + tanh(xTransition)*(dcoli - dSphere)/2.0;  // here tanh varies from (close to) -1 to 0
            
        }
        else
        {
            double xTransition = 2.0 * (npi-transitionFactor/2.0) / (transitionFactor/2.0);              // must vary from 0 to 2 for np from transitionFactor/2 to transitionFactor
            dci = (dcoli + dSphere) / 2.0 + tanh(xTransition)*(dcoli - dSphere)/2.0; // here tanh varies from 0 to (close to) 1
        }
        
    }
    else  // expression from fractal Dg
    {
        double kfi = kfInterpolation(i);
        double dg = dp * pow(npi/kfi, 1.0/df);  // in [cm]   expression from Dg = 2 Rg by fractal relation
        double dcoli = dg * pow(5.0/3.0, 0.5);  // Dc ~ 1.29 * Dg
        
        double dSphereNano = m_psdPtr->getdAvgNano(i);  // in [nm]
        double dSphere = dSphereNano * 1.0e-7; // in [cm]
        
        if(dcoli > dSphere)
        {
            dci = dcoli;
        }
    }
    return dci;  //  dc in [cm]
}


double SootSourceTerm::getDf()
{
    return m_Df;
}

double SootSourceTerm::freePath(double T, double P)  // P in Pa
{
    double lambda = 2.3701e-03 * T / P;  // from Berkeley MOM  (for air) (changed P in Pa instead of P in atm)
    return lambda; // in cm
}

double SootSourceTerm::knudsen(double T, double P, double dc)  // from Mauss KIT
{
    double lambda = freePath(T, P);   // in cm
    double kn = 2.0 * lambda / dc;    // ! units !
    return kn;
}

double SootSourceTerm::surfFromFractal(int bin)
{
    double surfAgglomerate(0.0);
    
    //double mass = m_psdPtr->getmAvg(bin);
    double mass = m_psdPtr->getmAvgVar(bin);
    
    double dpNano = m_imposedDpSource;
    double vPrimary = pi / 6.0 * pow(dpNano,3.0); // in [nm3]
    double vPrimarycm3 = vPrimary * 1.0e-21;   // in [cm3].
    double mPrimary = rhoSoot * vPrimarycm3;   // rhoSoot in [g/cm3]. mPrimary in [g]
    
    double np = mass / mPrimary;
    
    if(np<=1.0)
    {
        surfAgglomerate = m_psdPtr->getsAvg(bin);   // cm2. from sphere
    }
    else
    {
        double dcm = dpNano * 1.0e-7;  // in [cm]
        double surfPrimary = pi * pow(dcm, 2.0); // in [cm2]
        
        surfAgglomerate = np * surfPrimary;
    }
    
    return surfAgglomerate;  // in [cm2]
}

void SootSourceTerm::setSurfType(string surfType)
{
    m_surfType = surfType;
}

double SootSourceTerm::cunningham(double T, double P, double dc)  // from Mauss or Blanquart KIT
{
    double kn = knudsen(T, P, dc); // dc in [cm]
    double cunn = 1.0+1.257 * kn;
    return cunn;
}

double SootSourceTerm::dynVisco(double T)
{
    double C1 = 1.4558e-6; // [kg.m-1.s-1.K-1/2]
    double C2 = 110.4; // [K]
    double eta = C1*pow(T, 1.5) / (T+C2);
    return eta;
}


void SootSourceTerm::setMassTransferType(string massTransferType)
{
    m_massTransferType = massTransferType;
}

void SootSourceTerm::setAggloTensor(AgglomerationTensor *aggloTensorPtr)
{
    m_aggloTensorPtr = aggloTensorPtr;
}


double SootSourceTerm::tensorWmPosk(int kDestination, double T, double P) // [g/s/cm3]
{
    int i(0);
    int j(0);
    int nBins = m_psdPtr->getSize();
    
    double wmPosk(0.0);
    
    int maxjPlusOne = kDestination+2;  // j borné par k+1 et par binMax
    if(nBins<maxjPlusOne)
    {
        maxjPlusOne = nBins;
    }
    
    for(j=0; j<(maxjPlusOne); j++) // j <= k+1  (but could be j <= k because for j = k+1 gCoag = 0
    {
        for(i=0; i<(j+1); i++)   // i <= j
        {
            double wr = w_kj(i, j, T, P);  // // [collsions/s/cm3] collision frequency ~ betakj * nvj * nvk
            double mi = m_aggloTensorPtr->getmAvgFromiOrigin(i, j, kDestination);
            double mj = m_aggloTensorPtr->getmAvgFromjOrigin(i, j, kDestination);
            double gCoagk = m_aggloTensorPtr->getgCoagValue(i, j, kDestination);
            
            //cout << "i=" << i << "j=" << j << "k=" << kDestination << endl;
            //cout << "mi=" << mi << "mj=" << mj << "gCoag=" << gCoagk << endl;
            
            wmPosk += gCoagk * (mi + mj) * wr;  // [g/s/cm3] = [%] * [g/part] * [collisions/s/cm3]
        }
        //cout << endl << endl;
    }
    return wmPosk;
}

double SootSourceTerm::tensorWmNegi(int iOrigin, double T, double P)  // [g/s/cm3]. cases k = i <=j
{
    int j(0);
    int k(0);
    int nBins = m_psdPtr->getSize();
    
    double wmNegi(0.0);
    
    for(j=iOrigin; j<nBins; j++) // sum begins in iOrigins because j >= i
    {
        for(k=0; k<nBins; k++)
        {
            double wr = w_kj(iOrigin, j, T, P);  // // [collsions/s/cm3] collision frequency ~ betakj * nvj * nvk
            double mi = m_aggloTensorPtr->getmAvgFromiOrigin(iOrigin, j, k);
            double gCoag = m_aggloTensorPtr->getgCoagValue(iOrigin, j, k);
            
            if(j== iOrigin)
            {
                wmNegi -= gCoag * mi * wr;
            }
            else if(iOrigin < j)
            {
                wmNegi -= gCoag * mi * wr;  // [g/s/cm3] = [%] * [g/part] * [collisions/s/cm3]
            }
            
            
        }
        
    }
    return wmNegi;
}


double SootSourceTerm::tensorWmNegj(int jOrigin, double T, double P)  // [g/s/cm3]  cases k = j >i
{
    int i(0);
    int k(0);
    int nBins = m_psdPtr->getSize();
    
    double wmNegj(0.0);
    
    for(i=0; i<(jOrigin+1); i++)  // sum ends on jOrigin+1 because i <= j
    {
        for(k=0; k<nBins; k++)
        {
            double wr = w_kj(i, jOrigin, T, P);  // // [collsions/s/cm3] collision frequency ~ betakj * nvj * nvk
            double mj = m_aggloTensorPtr->getmAvgFromjOrigin(i, jOrigin, k);
            double gCoag = m_aggloTensorPtr->getgCoagValue(i, jOrigin, k);
            
            if(i == jOrigin)
            {
                wmNegj -= gCoag * mj * wr;
            }
            else if(i<jOrigin)
            {
                wmNegj -= gCoag * mj * wr;  // [g/s/cm3] = [%] * [g/part] * [collisions/s/cm3]
            }
            
        }
        
    }
    return wmNegj;
}


double SootSourceTerm::tensorWmTotk(int kDestination, double T, double P) // [g/s/cm3]
{
    double wmPos = tensorWmPosk(kDestination, T, P);
    double wmNegi = tensorWmNegi(kDestination, T, P);
    double wmNegj = tensorWmNegj(kDestination, T, P);
    
    double wmTot = wmPos + wmNegi + wmNegj; // wmPos + wmNegi + wmNegj ? (case i<=j separated from case i>j).
    //collision between i + j -> k ?
    
    //cout << "k=" << kDestination << endl;
    //cout << wmPos << "   " << wmNegi << "   " << wmNegj << endl;
    return wmTot;
}


// without T,P for solverType == "analytical"
double SootSourceTerm::tensorWmPosk(int kDestination) // [g/s/cm3]
{
    double wmPosk(0.0);
    
    if(m_solverType == "analytical")
    {
        int i(0);
        int j(0);
        int nBins = m_psdPtr->getSize();
        
        int maxjPlusOne = kDestination+2;  // j borné par k+1 et par binMax. j <= k+1. Why not j <= k? In practice if j = k+1 gcoag(i,j,k) will be equal to 0. Could substitute by maxjPlusOne = kDestination + 1
        if(nBins<maxjPlusOne)
        {
            maxjPlusOne = nBins;
        }
        
        for(j=0; j<(maxjPlusOne); j++) // j <= k+1
        {
            for(i=0; i<(j+1); i++)   // i <= j
            {
                double wr = w_kj(i, j);  // // [collsions/s/cm3] collision frequency ~ betakj * nvj * nvk
                double mi = m_aggloTensorPtr->getmAvgFromiOrigin(i, j, kDestination);
                double mj = m_aggloTensorPtr->getmAvgFromjOrigin(i, j, kDestination);
                double gCoagk = m_aggloTensorPtr->getgCoagValue(i, j, kDestination);
                
                //cout << "i=" << i << "j=" << j << "k=" << kDestination << endl;
                //cout << "mi=" << mi << "mj=" << mj << "gCoag=" << gCoagk << endl;
                
                wmPosk += gCoagk * (mi + mj) * wr;  // [g/s/cm3] = [%] * [g/part] * [collisions/s/cm3]
            }
            //cout << endl << endl;
        }
    }
    else
    {
        cout << "ERROR: solverType"<< endl;
        wmPosk = 0;
    }
    
    return wmPosk;
}

// without T,P for solverType == "analytical"
double SootSourceTerm::tensorWmNegi(int iOrigin)  // [g/s/cm3]. cases k = i <=j
{
    double wmNegi(0.0);
    
    if(m_solverType=="analytical")
    {
        int j(0);
        int k(0);
        int nBins = m_psdPtr->getSize();
        
        for(j=iOrigin; j<nBins; j++) // sum begins in iOrigins because j >= i
        {
            for(k=0; k<nBins; k++)
            {
                double wr = w_kj(iOrigin, j);  // // [collsions/s/cm3] collision frequency ~ betakj * nvj * nvk
                double mi = m_aggloTensorPtr->getmAvgFromiOrigin(iOrigin, j, k);
                double gCoag = m_aggloTensorPtr->getgCoagValue(iOrigin, j, k);
                
                if(j== iOrigin)
                {
                    wmNegi -= gCoag * mi * wr;
                }
                else if(iOrigin < j)
                {
                    wmNegi -= gCoag * mi * wr;  // [g/s/cm3] = [%] * [g/part] * [collisions/s/cm3]
                }
                
                
            }
            
        }
    }
    else
    {
        cout << "ERROR: solverType" << endl;
    }
    
    return wmNegi;
}

// without T,P for solverType == "analytical"
double SootSourceTerm::tensorWmNegj(int jOrigin)  // [g/s/cm3]  cases k = j >i
{
    double wmNegj(0.0);
    
    if(m_solverType=="analytical")
    {
        int i(0);
        int k(0);
        int nBins = m_psdPtr->getSize();
        
        for(i=0; i<(jOrigin+1); i++)  // sum ends on jOrigin+1 because i <= j
        {
            for(k=0; k<nBins; k++)
            {
                double wr = w_kj(i, jOrigin);  // // [collsions/s/cm3] collision frequency ~ betakj * nvj * nvk
                double mj = m_aggloTensorPtr->getmAvgFromjOrigin(i, jOrigin, k);
                double gCoag = m_aggloTensorPtr->getgCoagValue(i, jOrigin, k);
                
                if(i == jOrigin)
                {
                    wmNegj -= gCoag * mj * wr;
                }
                else if(i<jOrigin)
                {
                    wmNegj -= gCoag * mj * wr;  // [g/s/cm3] = [%] * [g/part] * [collisions/s/cm3]
                }
                
            }
            
        }
    }
    else
    {
        cout << "ERROR: solverType"<<endl;
        
    }
    
    return wmNegj;
}

// without T,P for solverType == "analytical"
double SootSourceTerm::tensorWmTotk(int kDestination) // [g/s/cm3]
{
    double wmTot(0.0);
    
    if(m_solverType=="analytical")  // collision i + j -> k  with i <= j and j <= k
    {
        double wmPos = tensorWmPosk(kDestination);
        double wmNegi = tensorWmNegi(kDestination);
        double wmNegj = tensorWmNegj(kDestination);
        
        wmTot = wmPos + wmNegi + wmNegj; // wmPos + wmNegi + wmNegj ? (case i<=j separated from case i>j)
    }
    else
    {
        cout << "ERROR: solverType" <<endl;
    }
    
    return wmTot;
}

// for m_solverType == "analytical". Another function depending on T and P is used for m_solverType == "HomogeneousGasPhase"
// ref Kumar and Ramkrishna 1996: eq (36), (39) and (41) or Park 2004 eq (2), (3)
double SootSourceTerm::kumarAggloSource(int iDestination) // collision k <= j -> i
{
    double dotNi(0.0); // [part/cm3/s]
    double dotNiPos(0.0); // [part/cm3/s]
    double dotNiNeg(0.0); // [part/cm3/s]
    
    int nBins = m_psdPtr->getSize();
    int k(0);
    int j(0);
    
    for(j = 0; j < (iDestination+1); j++ )  // j <= iDestination
    {
        for(k = 0; k < (j+1); k++ )  // k <= j
        {
            double delta_jk(0.0);
            if( j == k)
            {
                delta_jk = 1.0;
            }
            
            double eta = kumarEta(k, j, iDestination);
            //cout << eta << endl;
            
            double beta_jk(0.0);
            
            if(m_solverType == "analytical")
            {
                beta_jk = betaAggloAnalytical(k, j);
            }
            else
            {
                cout << "ERROR solverType in kumarAggloSource" << endl;
            }
            
            double Nj = m_psdPtr->getNv(j);
            double Nk = m_psdPtr->getNv(k);
            
            // equation for dotNiPos:
            dotNiPos += ( 1- delta_jk/2.0 ) * eta * beta_jk * Nj * Nk;
            //cout << "Nj = " << Nj << " Nk = " << Nk << endl;
            //cout << "delta_jk = " << delta_jk << "   beta_jk = " << beta_jk << "eta = " << eta << endl;
        }
        
    }
    //cout << "section: " << iDestination << "   dotNiPos = " << dotNiPos << endl;
    
    for(k=0; k<nBins; k++)
    {
        double beta_ik(0.0);
        
        if(m_solverType == "analytical")
        {
            beta_ik = betaAggloAnalytical(iDestination, k);
        }
        else
        {
            cout << "ERROR solverType in kumarAggloSource" << endl;
        }
        
        double Ni = m_psdPtr->getNv(iDestination);
        double Nk = m_psdPtr->getNv(k);
        
        // equation for dotNiNeg
        dotNiNeg += -Ni * beta_ik * Nk;
    }
    dotNi = dotNiNeg + dotNiPos;
    //cout << endl << "section: " << iDestination  << "   dotNiNeg = " << dotNiNeg << "   dotNiPos = " << dotNiPos << endl;
    
    return dotNi;
}

// for m_solverType == "HomogeneousGasPhase"
// ref Kumar and Ramkrishna 1996: eq (36), (39) and (41) or Park 2004 eq (2), (3)
double SootSourceTerm::kumarAggloSource(int iDestination, double T, double P)
{
    double dotNi(0.0); // [part/cm3/s]
    double dotNiPos(0.0); // [part/cm3/s]
    double dotNiNeg(0.0); // [part/cm3/s]
    
    int nBins = m_psdPtr->getSize();
    int k(0);
    int j(0);
    
    for(j = 0; j < (iDestination+1); j++ )  // j <= iDestination
    {
        for(k = 0; k < (j+1); k++ )  // k <= j
        {
            double delta_jk(0.0);
            if( j == k)
            {
                delta_jk = 1.0;
            }
            
            double eta = kumarEta(k, j, iDestination);
            
            double beta_jk(0.0);
            
            if(m_solverType == "HomogeneousGasPhase")
            {
                beta_jk = betaAggloHomogeneousGasPhase(k, j, T, P);
            }
            else
            {
                cout << "ERROR solverType in kumarAggloSource" << endl;
            }
            
            double Nj = m_psdPtr->getNv(j);
            double Nk = m_psdPtr->getNv(k);
            
            // equation for dotNiPos:
            dotNiPos += ( 1- delta_jk/2.0 ) * eta * beta_jk * Nj * Nk;
        }
    }
    
    for(k=0; k<nBins; k++)
    {
        double beta_ik(0.0);
        
        if(m_solverType == "HomogeneousGasPhase")
        {
            beta_ik = betaAggloHomogeneousGasPhase(iDestination, k, T, P);
            
        }
        else
        {
            cout << "ERROR solverType in kumarAggloSource" << endl;
        }
        
        double Ni = m_psdPtr->getNv(iDestination);
        double Nk = m_psdPtr->getNv(k);
        
        // equation for dotNiNeg
        dotNiNeg += -Ni * beta_ik * Nk;
        
        
    }
    dotNi = dotNiNeg + dotNiPos;
    
    //cout << "dotNi =  " << dotNi << "  dotNiNeg = " << dotNiNeg << "  dotNiPos = " << dotNiPos << endl;
    
    return dotNi;
}

double SootSourceTerm::kumarEta(int k, int j, int iDestination)
{
    double eta(0.0);
    double nBins = m_psdPtr->getSize();
    //double mAvgk = m_psdPtr->getmAvg(k);
    //double mAvgj = m_psdPtr->getmAvg(j);
    //double mAvgi = m_psdPtr->getmAvg(iDestination); // !! for idestination mAvgConst is kept !!
    
    double mAvgVark = m_psdPtr->getmAvgVar(k);
    double mAvgVarj = m_psdPtr->getmAvgVar(j);
    double mAvgVari = m_psdPtr->getmAvgVar(iDestination);
    
    if(iDestination == 0) // then mAvgk + mAvgj >= vi
    {
        //double mAvgiPlusOne = m_psdPtr->getmAvg(iDestination+1); // !! for idestination mAvgConst is kept !!
        
        double mAvgVariPlusOne = m_psdPtr->getmAvgVar(iDestination+1);
        
        if( (mAvgVarj + mAvgVark) <= mAvgVariPlusOne )
        //if( (mAvgVarj + mAvgVark) <= mAvgiPlusOne )
        {
            eta = ( mAvgVariPlusOne - (mAvgVarj + mAvgVark) ) / ( mAvgVariPlusOne - mAvgVari );
            //eta = ( mAvgiPlusOne - (mAvgVarj + mAvgVark) ) / ( mAvgiPlusOne - mAvgi );
        }
        else
        {
            eta = 0.0;
        }
    }
    else if(iDestination == (nBins - 1 )) // then mAvgk + mAvgj <= mAvgi or no bigger bin to go -> accumulation in last bin
    {
        //double mAvgiMinusOne = m_psdPtr->getmAvg(iDestination - 1);
        double mAvgVariMinusOne = m_psdPtr->getmAvgVar(iDestination - 1);
        
        if( ( mAvgVarj + mAvgVark ) >= mAvgVariMinusOne )
        //if( ( mAvgVarj + mAvgVark ) >= mAvgiMinusOne )
        {
            
            eta = ( mAvgVariMinusOne - ( mAvgVarj + mAvgVark ) ) / ( mAvgVariMinusOne - mAvgVari );
            //eta = ( mAvgiMinusOne - ( mAvgVarj + mAvgVark ) ) / ( mAvgiMinusOne - mAvgi );
        }
        else
        {
            eta = 0.0;
        }
    }
    else
    {
        //double mAvgiMinusOne = m_psdPtr->getmAvg(iDestination - 1);
        //double mAvgiPlusOne = m_psdPtr->getmAvg(iDestination+1);
        double mAvgVariMinusOne = m_psdPtr->getmAvgVar(iDestination - 1);
        double mAvgVariPlusOne = m_psdPtr->getmAvgVar(iDestination+1);
        
        if( ((mAvgVarj + mAvgVark) <= mAvgVariPlusOne) && ((mAvgVarj + mAvgVark) >= mAvgVari) )
        //if( ((mAvgVarj + mAvgVark) <= mAvgiPlusOne) && ((mAvgVarj + mAvgVark) >= mAvgi) )
        {
            
            eta = ( mAvgVariPlusOne - (mAvgVarj + mAvgVark) ) / ( mAvgVariPlusOne - mAvgVari );
            //eta = ( mAvgiPlusOne - (mAvgVarj + mAvgVark) ) / ( mAvgiPlusOne - mAvgi );
        }
        else if( (( mAvgVarj + mAvgVark ) >= mAvgVariMinusOne) &&  (( mAvgVarj + mAvgVark ) <= mAvgVari) )
        //else if( (( mAvgVarj + mAvgVark ) >= mAvgiMinusOne) &&  (( mAvgVarj + mAvgVark ) <= mAvgi) )
        {
            
            eta = ( mAvgVariMinusOne - ( mAvgVarj + mAvgVark ) ) / ( mAvgVariMinusOne - mAvgVari );
            //eta = ( mAvgiMinusOne - ( mAvgVarj + mAvgVark ) ) / ( mAvgiMinusOne - mAvgi );
        }
        else
        {
            eta = 0.0;
        }
        
    }
    
    return eta;
}

double SootSourceTerm::betaAggloAnalytical(int k, int j)
{
    double betakj(0.0);
    
    if(m_solverType == "analytical")
    {
        if(m_aggloType == "betaConst")
        {
            betakj = m_betaConst;
        }
        else if (m_aggloType == "sumKernel")
        {
            double mAvgk = m_psdPtr->getmAvg(k);
            double mAvgj = m_psdPtr->getmAvg(j);
            
            betakj = m_betaConst * (mAvgk + mAvgj);
            
        }
        else
        {
            cout << "ERROR: aggloType" << endl;
        }
    }
    else
    {
        cout << "ERROR: solverType"<< endl;
    }
    return betakj;
}

double SootSourceTerm::betaAggloHomogeneousGasPhase(int k, int j, double T, double P)
{
    double betakj(0.0);
    
    if(m_solverType == "HomogeneousGasPhase")
    {
        double mAvgj = m_psdPtr->getmAvg(j);
        double mAvgk = m_psdPtr->getmAvg(k);
        
        if(m_aggloType == "standard")
        {
            betakj = betaFree(mAvgj, mAvgk, T);
        }
        else if (m_aggloType == "collisionDiameter")
        {
            double dcj = collisionDiameterCalc(mAvgj);
            double dck = collisionDiameterCalc(mAvgk);
            
            betakj = betaFreeDc(mAvgj, mAvgk, dcj, dck, T);
        }
        else if(m_aggloType == "betaConst")
        {
            betakj = m_betaConst;
        }
        else if (m_aggloType == "sumKernel")
        {
            double mAvgk = m_psdPtr->getmAvg(k);
            double mAvgj = m_psdPtr->getmAvg(j);
            
            betakj = m_betaConst * (mAvgk + mAvgj);
        }
        else if( m_aggloType == "harmonicAvgDc")
        {
            
            double dcj = collisionDiameterCalc(mAvgj);
            double dck = collisionDiameterCalc(mAvgk);
            
            betakj = betaHarmonicDc(mAvgj, mAvgk, dcj, dck, T, P);
        }
        else
        {
            cout << "problem m_aggloType" << endl;
            betakj = 0.0;
            
        }
    }
    else
    {
        cout << "ERROR: solverType"<< endl;
    }
    
    return betakj;
}
